@@ -1,4 +1,4 @@
-/* $Id: host_protocol.c,v 1.104 2004/11/29 22:42:57 graziano Exp $ */
+/* $Id: host_protocol.c,v 1.117 2005/08/04 00:18:31 graziano Exp $ */
 
 #include "config_nws.h"
 #include <stdio.h>      /* FILE sprintf() rewind() */
@@ -77,6 +77,51 @@ TermHandler(int sig) {
 	exit(2);
 }
 
+/* get a new nameservers list from an open connection with a nameserver. 
+ * The nameserver is assumed in the cookie.
+ * Return 1 on success, 0 otherwise
+ */
+static int 
+NewNameServerList(struct host_cookie *cookie) {
+	ObjectSet tmpSet;
+	int ret = 0;
+
+	/* nameservers don't get nameservers here */
+	if (myType == NAME_SERVER_HOST) {
+		return 1;
+	}
+
+	/* sanitcy check */
+	if (cookie == NULL) {
+		ERROR("NewNameServerList: NULL parameter\n");
+		return 0;
+	}
+
+	/* let's try to get the new list */
+	if (RetrieveFromMyNameserver("&(objectclass=nwsHost)(hostType=nameserver)", &tmpSet)) {
+		/* let's check we got some nameservers */
+		GetNWSLock(&lock);
+		if (tmpSet[0] != '\0') {
+			FREE(nameServers);
+			nameServers = tmpSet;
+			ret = 1;
+		} else {
+			/* no good */
+			FREE(tmpSet);
+		}
+		ReleaseNWSLock(&lock);
+	}
+
+	/* some info in the log */
+	if (ret) {
+		INFO1("NewNameServerList: new nameservers list (%s)\n", tmpSet);
+	} else {
+		WARN("NewNameServerList: failed to get  new nameservers list\n");
+	}
+
+	return ret;
+}
+
 
 /*
  * A "local" function of EstablishHost().  Handles a #messageType# message
@@ -110,7 +155,7 @@ ProcessRequest(Socket *sd,
 		break;
 
 	case HOST_DIE:
-		dataString = (char *)MALLOC(header.dataSize + 1);
+		dataString = (char *)MALLOC(header.dataSize + 1, 0);
 		if (dataString == NULL) {
 			ERROR("ProcessRequest: out of memory\n");
 			break;
@@ -143,7 +188,7 @@ ProcessRequest(Socket *sd,
 		 * for example, of the new name server.
 		 */
 		nsDescriptor.repetitions = header.dataSize;
-		dataString = (char *)MALLOC(header.dataSize + 1);
+		dataString = (char *)MALLOC(header.dataSize + 1, 0);
 		if (dataString == NULL) {
 			ERROR("ProcessRequest: out of memory\n");
 			break;
@@ -162,6 +207,9 @@ ProcessRequest(Socket *sd,
 
 				/* drop old socket */
 				DROP_SOCKET(&myNameServer.sd);
+
+				/* get the new list of nameservers */
+				NewNameServerList(&myNameServer);
 			}
 			ReleaseNWSLock(&lock);
 
@@ -177,7 +225,7 @@ ProcessRequest(Socket *sd,
 		 * We can then change the memory here.
 		 */
 		nsDescriptor.repetitions = header.dataSize;
-		dataString = (char *)MALLOC(header.dataSize + 1);
+		dataString = (char *)MALLOC(header.dataSize + 1, 0);
 		if (dataString == NULL) {
 			ERROR("ProcessRequest: out of memory\n");
 			break;
@@ -330,7 +378,7 @@ CreateHostObject(	const char *myName,
 			unsigned int addressesCount,
 			unsigned short port) {
 	const char *HOST_TYPES[] = {"forecaster", "memory", 
-			"nameserver", "sensor"};
+			"nameserver", "sensor", "proxy"};
 	const char *MONTH_NAMES[] =
 			{"Jan", "Feb", "Mar", "Apr", "May", "Jun",
 			"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
@@ -439,7 +487,7 @@ EstablishHost(const char *registration,
 	/* we piss all over the local variable: we need a big lock here.
 	 * Thankfully we call this function only once ... */
 	GetNWSLock(&lock);
-	myAddresses = (IPAddress *)MALLOC(addressesCount * sizeof(IPAddress));
+	myAddresses = (IPAddress *)MALLOC(addressesCount * sizeof(IPAddress), 1);
 	myRegistration = strdup(registration);
 	myPassword = strdup((password == NULL) ? "" : password);
 	if((myAddresses == NULL) || (myRegistration == NULL) || (myPassword == NULL)){
@@ -505,10 +553,7 @@ EstablishHost(const char *registration,
 	hostObject = CreateHostObject(registration, hostType, addresses, addressesCount, port);
 
 	/* let's tell which flags have been used for this compilation */
-	tmp = MALLOC(strlen("experimental") + strlen("debug") + strlen("thread") + 5);
-	if (tmp == NULL) {
-		ABORT("EstablishHost: out of memory\n");
-	}
+	tmp = MALLOC(strlen("experimental") + strlen("debug") + strlen("thread") + 5, 1);
 	tmp[0] = '\0';
 
 #ifdef EXPERIMENTAL
@@ -631,7 +676,10 @@ RetrieveFromMyNameserver(	const char *filter,
 		SAFESTRCPY(myNameServer.name, cookie.name);
 		myNameServer.port = cookie.port;
 		myNameServer.sd = cookie.sd;
+		DROP_SOCKET(&myNameServer.sd);
 	} else {
+		WARN("RetrieveFromMyNameserver: can't talk with nameserver\n");
+
 		/* we couldn't connect */
 		myNameServer.sd = NO_SOCKET;
 	}
@@ -657,10 +705,7 @@ HostHealthy(void) {
 
 	/* let's create the filter */
 	GetNWSLock(&lock);
-	filter = (char *)MALLOC(LOOKUP_FILTER_LEN + strlen(myRegistration));
-	if (filter == NULL) {
-		ABORT("HostHealthy: out of memory\n");
-	}
+	filter = (char *)MALLOC(LOOKUP_FILTER_LEN + strlen(myRegistration), 1);
 	sprintf(filter, LOOKUP_FILTER, myRegistration);
 	ReleaseNWSLock(&lock);
 	if (RetrieveFromMyNameserver(filter, &hostObject)) {
@@ -685,7 +730,6 @@ HostHealthy(void) {
  * saving the values. We keep some backup of old values we couldn't store
  * into the memory.  Returns 1 if we got a backup of the eperiment, 0
  * otherwise. */
-#define KEEP_A_LONG_TIME 315360000.0
 #define BACKUPLENGTH 512
 int
 RegisterExperiment(	char *registration,
@@ -712,7 +756,7 @@ RegisterExperiment(	char *registration,
 		return 0;
 	}
 	length = tmp - registration + OBJECT_TERMINATOR_LEN;
-	tmp = MALLOC(length + 1);
+	tmp = MALLOC(length + 1, 0);
 	if (tmp == NULL) {
 		ERROR("RegisterExperiment: out of memory\n");
 		return 0;
@@ -734,10 +778,7 @@ RegisterExperiment(	char *registration,
 	backupSeries[backupExpEnd] = tmp;
 
 	/* save the experiment in the backup queue */
-	backupExps[backupExpEnd] = (NWSAPI_Measurement *)MALLOC(sizeof(NWSAPI_Measurement));
-	if (backupExps[backupExpEnd] == NULL) {
-		ABORT("RegisterExperiment: out of memory\n");
-	}
+	backupExps[backupExpEnd] = (NWSAPI_Measurement *)MALLOC(sizeof(NWSAPI_Measurement), 1);
 	backupExps[backupExpEnd]->timeStamp = experiment->timeStamp;
 	backupExps[backupExpEnd]->measurement = experiment->measurement;
 
@@ -773,51 +814,6 @@ RegisterExperiment(	char *registration,
 	return 1;
 }
 
-/* get a new nameservers list from an open connection with a nameserver. 
- * The nameserver is assumed in the cookie.
- * Return 1 on success, 0 otherwise
- */
-static int 
-NewNameServerList(struct host_cookie *cookie) {
-	ObjectSet tmpSet;
-	int ret = 0;
-
-	/* nameservers don't get nameservers here */
-	if (myType == NAME_SERVER_HOST) {
-		return 1;
-	}
-
-	/* sanitcy check */
-	if (cookie == NULL) {
-		ERROR("NewNameServerList: NULL parameter\n");
-		return 0;
-	}
-
-	/* let's try to get the new list */
-	if (RetrieveFromMyNameserver("&(objectclass=nwsHost)(hostType=nameserver)", &tmpSet)) {
-		/* let's check we got some nameservers */
-		GetNWSLock(&lock);
-		if (tmpSet[0] != '\0') {
-			FREE(nameServers);
-			nameServers = tmpSet;
-			ret = 1;
-		} else {
-			/* no good */
-			FREE(tmpSet);
-		}
-		ReleaseNWSLock(&lock);
-	}
-
-	/* some info in the log */
-	if (ret) {
-		INFO1("NewNameServerList: new nameservers list (%s)\n", tmpSet);
-	} else {
-		WARN("NewNameServerList: failed to get  new nameservers list\n");
-	}
-
-	return ret;
-}
-
 /* register all the object with the best nameserver we can. 
  * On success returns 1 and set the good nameserver in myNameServer. */
 int 
@@ -902,7 +898,6 @@ RegisterHost(unsigned long timeOut) {
 		NewNameServerList(&myNameServer);
 		ret = 1;
 	}
-	DisconnectHost(&myNameServer);	/* we don't cache sockets here */
 	FREE(tmp);
 
   	return ret;
@@ -913,42 +908,26 @@ void
 RegisterObject(const Object object) {
 	short tmp;
 	char *name;
-	Object nextObj;
+
+	/* first of all let's unregister and remove the old object */
+	name = NwsAttributeValue_r(FindNwsAttribute(object, "name"));
+	if (name != NULL) {
+		/* we don't want to go to the nameserver, since
+		 * we'll already override the name */
+		DirectUnregisterObject(name, 0);
+	}
+	FREE(name);
 
 	/* we need to lock all around registrationSet */
 	GetNWSLock(&lock);
-
-	/* add the object to the objectSet */
 	if (registrationSet == NO_OBJECT_SET) {
 		registrationSet = NewObjectSet();
 	}
-
-	tmp = 0;
-
-	/* if we don't have the object, or the one we have is different
-	 * (more recent), substitute it */
-	nextObj = NextObject(registrationSet, NULL);
-	for (; nextObj != NO_OBJECT; nextObj = NextObject(registrationSet, nextObj)) {
-		if (AreObjectsEquivalent(nextObj, object)) {
-			/* they are the same */
-			break;
-		}
-	}
-	if (nextObj == NO_OBJECT) {
-		/* substitute if we got a name for the object */
-		name = NwsAttributeValue_r(FindNwsAttribute(object, "name"));
-		if (name != NULL) {
-			/* we don't want to go to the nameserver, since
-			 * we'll already override the name */
-			DirectUnregisterObject(name, 0);
-		}
-		FREE(name);
-		tmp = AddObject(&registrationSet, object);
-	}
+	tmp = AddObject(&registrationSet, object);
 	ReleaseNWSLock(&lock);
 
 	if (tmp) {
-		RegisterWithNameServer(object, DEFAULT_HOST_BEAT);
+		RegisterWithNameServer(object, DEFAULT_HOST_BEAT * 2);
 	}
 	
 	/* we don't disconnect here: mainly at the beginning we call a
@@ -1000,8 +979,6 @@ DirectUnregisterObject(	const char *objectName,
 	char *filter;
 	int len;
 	NwsAttribute nameAttr;
-	ObjectSet thinnedSet = NO_OBJECT_SET;
-	struct host_cookie cookie;
 
 	/* sanity check */
 	if (objectName == NULL) {
@@ -1016,62 +993,30 @@ DirectUnregisterObject(	const char *objectName,
 		return;
 	}
 
-	/* compute the lenght of this filter */
-	len = strlen(objectName) + 10;
-	filter = (char *)MALLOC(sizeof(char)*len);
-	if (filter == NULL) {
-		ABORT("UnregisterObject: out of memory\n");
-	}
-	vstrncpy(filter, len, 3, "&(name=", objectName, ")");
-
-	/* we don't query the nameserver is force is not enabled */
+	/* we don't query the nameserver if force is not enabled */
 	if (force) {
-		/* we need to lock everything since the nameserver can
-		 * change along the way */
-		GetNWSLock(&lock);
-
-		/* let's try the canonical nameserver first */
-		cookie.sd = NO_SOCKET;
-		if (!Unregister(&myNameServer, filter)) {
-			/* let's try to send to all the nameservers we are aware of  */
-			for (current = NextObject(nameServers, NULL); current != NULL; current = NextObject(nameServers, current)) {
-				if (ConnectToObject(current, &cookie)) {
-					/* now let's try to register */
-					if (Unregister(&cookie, filter)) {
-						SAFESTRCPY(myNameServer.name, cookie.name);
-						myNameServer.port = cookie.port;
-						myNameServer.sd = cookie.sd;
-						break;
-					}
-				}
-			}
-		}
-		DisconnectHost(&myNameServer);
+		/* let's generate a filter and unregister it */
+		len = strlen(objectName) + 10;
+		filter = (char *)MALLOC(sizeof(char)*len, 1);
+		vstrncpy(filter, len, 3, "&(name=", objectName, ")");
+		UnregisterFilter(filter);
+		FREE(filter);
 	}
-	FREE(filter);
 
 	/* let's look for objects with this name */
+	GetNWSLock(&lock);
 	for(current = NextObject(registrationSet, NO_OBJECT); current != NO_OBJECT; current = NextObject(registrationSet, current)) {
 		nameAttr = NwsAttributeValue_r(FindNwsAttribute(current, "name"));
-		if(nameAttr != NO_ATTRIBUTE && strcmp(nameAttr, objectName)) {
-			if (thinnedSet == NO_OBJECT_SET) {
-				/* let's create the object */
-				thinnedSet = NewObjectSet();
-			}
-
-			/* it's not the one we are looking for */
-			AddObject(&thinnedSet, current);
+		if (nameAttr != NO_ATTRIBUTE && !strcmp(nameAttr, objectName)) {
+			/* we got it: let's remove the object */
+			DeleteObject(&registrationSet, current);
+			
+			/* registrationSet might have changed: let's
+			 * reinitialize current */
+			current = NextObject(registrationSet, NO_OBJECT);
 		}
 		FREE(nameAttr);
 	}
- 
-	/* if we got something in thinnedSet we need to change
-	 * registrationSet */
-	if (thinnedSet != NO_OBJECT_SET) {
-		FreeObjectSet(&registrationSet);
-		registrationSet = thinnedSet;
-	}
-
 	ReleaseNWSLock(&lock);
 }
 
@@ -1080,6 +1025,46 @@ UnregisterObject(const char *name) {
 	DirectUnregisterObject(name, 1);
 }
 
+void
+UnregisterFilter(const char *filter) {
+	Object current;
+	struct host_cookie cookie;
+
+	/* sanity check */
+	if (filter == NULL || filter[0] == '\0') {
+		ERROR("UnregisterFilter: NULL parameter!\n");
+		return;
+	}
+
+	/* we need to lock everything since the nameserver can
+	 * change along the way */
+	GetNWSLock(&lock);
+
+	/* let's try the canonical nameserver first */
+	cookie.sd = NO_SOCKET;
+	if (!Unregister(&myNameServer, filter)) {
+		/* let's try to send to all the nameservers we are aware of  */
+		for (current = NextObject(nameServers, NULL); current != NULL; current = NextObject(nameServers, current)) {
+			if (ConnectToObject(current, &cookie)) {
+				/* now let's try to register */
+				if (Unregister(&cookie, filter)) {
+					SAFESTRCPY(myNameServer.name, cookie.name);
+					myNameServer.port = cookie.port;
+					myNameServer.sd = cookie.sd;
+					break;
+				}
+			}
+		}
+	}
+	/* we should disconect here, but we let the socket open:
+	 * in case we are exiting it will speed up the exit, and
+	 * in a simple case of unregistration, there will be a
+	 * Register() just right after which will close the
+	 * socket */
+	/*DisconnectHost(&myNameServer);*/
+	ReleaseNWSLock(&lock);
+}
+
 
 ObjectSet
 RegistrationSet(void) {
@@ -1288,7 +1273,7 @@ HostGetRegistration(	struct host_cookie *cookie,
 	}
 
 	objDescriptor.repetitions = dataSize;
-	*objs = (char *)MALLOC(dataSize + 1);
+	*objs = (char *)MALLOC(dataSize + 1, 0);
 	if (*objs == NULL) {
 		ERROR("HostChangeMemory: out of memory\n");
 		return 0;
