@@ -1,4 +1,4 @@
-/* $Id: register.c,v 1.104 2004/11/09 22:24:06 graziano Exp $ */
+/* $Id: register.c,v 1.111 2005/05/24 19:38:25 graziano Exp $ */
 
 #include "config_nws.h"
 
@@ -18,16 +18,6 @@
 ** object set is simply a list of these.
 */
 
-/*
-** Values returned from NwsAttribute{Name,Value}.  We switched from static, fixed-
-** size arrays to allocated items because the potential length (of values, at
-** least) is unbounded.  This could cause a memory leak, but, since attributes
-** only exist within objects, we can be assured of freeing the memory by
-** checking the variable values within Free{Object,ObjectSet}.
-*/
-static char *currentName = NULL;
-static char *currentValue = NULL;
-
 /* it should be thread safe (given you are not overlapping memory used by
  * the parameters) */
 void
@@ -50,10 +40,7 @@ AddNwsAttribute(	Object *toObject,
 	expandedLen = objectLen + strlen(separator) + strlen(addName) +
 			(NAME_TERMINATOR_LEN) + strlen(addValue) + 1;
 
-	expanded = (Object)REALLOC(*toObject, expandedLen);
-	if(expanded == NULL) {
-		ABORT("AddNwsAttribute: out of memory\n");
-	}
+	expanded = (Object)REALLOC(*toObject, expandedLen, 1);
 	
 	/* Overwrite the existing object terminator. */
 	vstrncpy(expanded + objectLen - (OBJECT_TERMINATOR_LEN),
@@ -107,7 +94,7 @@ DeleteNwsAttribute(	Object *fromObject,
 		return 0;
 	}
 	memmove(tmp, tmp1+1, strlen(tmp1+1)+1);	/* copy the \0 too */
-	shrunk = (Object)REALLOC(*fromObject, sizeof(char)*(strlen(*fromObject)+1));
+	shrunk = (Object)REALLOC(*fromObject, sizeof(char)*(strlen(*fromObject)+1), 0);
 	if (shrunk == NULL) {
 		ERROR("DeleteNwsAttribute: realloc failed\n");
 		return 0;
@@ -159,14 +146,9 @@ AreObjectsEquivalent(	const Object lhs,
 	return done;
 }
 
-/*
- * add an object to the the object set iff it's not already there.
- * Returns 1 upon success 0 otherwise.
- */
 int
-AddObject(ObjectSet *toSet,
-          const Object addObject) {
-
+AddObject(	ObjectSet *toSet,
+		const Object addObject) {
 	const char *endObject;
 	ObjectSet expanded;
 	size_t objectLen;
@@ -186,11 +168,7 @@ AddObject(ObjectSet *toSet,
 	objectLen = endObject - addObject + OBJECT_TERMINATOR_LEN;
 
 	/* let's go on and add the object */
-	expanded = (ObjectSet)REALLOC(*toSet, setLen + objectLen + 1);
-	if(expanded == NULL) {
-		ERROR("AddObject: out of memory\n");
-		return 0;
-	}
+	expanded = (ObjectSet)REALLOC(*toSet, setLen + objectLen + 1, 1);
 
 	strncat(expanded, addObject, objectLen);
 	*toSet = expanded;
@@ -198,6 +176,46 @@ AddObject(ObjectSet *toSet,
 	return 1;
 }
 
+int
+DeleteObject(	ObjectSet *fromSet,
+		const Object delObject) {
+	Object obj;
+	ObjectSet thinned;
+	int done;
+
+	/* sanity check */
+	if (fromSet == NULL || *fromSet == NULL) {
+		ERROR("DeleteObject: NULL paramter\n");
+		return 0;
+	}
+
+	/* let's see if we can find the object */
+	thinned = NewObjectSet();
+	done = 0;
+	for (obj = NextObject(*fromSet, NULL); obj != NULL; obj = NextObject(*fromSet, obj)) {
+		if (!AreObjectsEquivalent(obj, delObject)) {
+			if (!AddObject(&thinned, obj)) {
+				ERROR("DeleteObject: failed to copy object\n");
+				FreeObjectSet(&thinned);
+				return 0;
+			}
+			continue;
+		} else {
+			/* we found the object to delete */
+			done = 1;
+		}
+	}
+	if (!done) {
+		FreeObjectSet(&thinned);
+		DDEBUG("DeleteObject: no object to delete have been found\n");
+		done = 0;
+	} else {
+		FreeObjectSet(fromSet);
+		*fromSet = thinned;
+	}
+
+	return done;
+}
 
 
 char *
@@ -214,13 +232,9 @@ NwsAttributeName_r(const NwsAttribute ofNwsAttribute) {
 
 	if(endOfName != NULL) {
 		nameLen = endOfName - ofNwsAttribute;
-		name = (char *)MALLOC(nameLen + 1);
-		if(name == NULL) {
-			ABORT("NwsAttributeName: out of memory\n");
-		} else {
-			memcpy(name, ofNwsAttribute, nameLen);
-			name[nameLen] = '\0';
-		}
+		name = (char *)MALLOC(nameLen + 1, 1);
+		memcpy(name, ofNwsAttribute, nameLen);
+		name[nameLen] = '\0';
 	}
 	return(name);
 }
@@ -250,13 +264,9 @@ NwsAttributeValue_r(const NwsAttribute ofNwsAttribute) {
 	}
 
 	valueLen = endOfValue - endOfName - 1;
-	value = (char *)MALLOC(valueLen + 1);
-	if(value == NULL) {
-		ABORT("NwsAttributeValue_r: out of memory\n");
-	} else {
-		memcpy(value, endOfName + 1, valueLen);
-		value[valueLen] = '\0';
-	}
+	value = (char *)MALLOC(valueLen + 1, 1);
+	memcpy(value, endOfName + 1, valueLen);
+	value[valueLen] = '\0';
 
 	return(value);
 
@@ -674,7 +684,7 @@ RetrieveObjects(struct host_cookie *whoFrom,
 		FAIL("RetrieveObjects: message receive failed\n");
 	}
 
-	filtered = (ObjectSet)MALLOC(replySize + 1);
+	filtered = (ObjectSet)MALLOC(replySize + 1, 0);
 	if(filtered == NULL) {
 		FAIL("RetrieveObjects: out of memory\n");
 	}
@@ -725,72 +735,6 @@ Unregister(struct host_cookie *withWho,
 }
 
 
-/* DEPRECATED: the following functions are not thread safe: use the _r
- * version */
-const char *
-NwsAttributeName(const NwsAttribute ofNwsAttribute) {
-
-	const char *endOfName;
-	size_t nameLen;
-
-	/* sanity check */
-	if (ofNwsAttribute == NULL)
-		return NULL;
-
-  if(currentName != NULL) {
-    FREE(currentName);
-  }
-
-  endOfName = strstr(ofNwsAttribute, NAME_TERMINATOR);
-
-  if(endOfName != NULL) {
-    nameLen = endOfName - ofNwsAttribute;
-    currentName = (char *)MALLOC(nameLen + 1);
-    if(currentName != NULL) {
-      memcpy(currentName, ofNwsAttribute, nameLen);
-      currentName[nameLen] = '\0';
-    }
-  }
-
-  return(currentName);
-
-}
-const char *
-NwsAttributeValue(const NwsAttribute ofNwsAttribute) {
-
-	const char *endOfName;
-	const char *endOfValue;
-	size_t valueLen;
-
-	/* sanity check */
-	if (ofNwsAttribute == NULL)
-		return NULL;
-
-  if(currentValue != NULL) {
-    FREE(currentValue);
-  }
-
-  endOfName = strstr(ofNwsAttribute, NAME_TERMINATOR);
-  if(endOfName == NULL) {
-    return NULL;
-  }
-
-  endOfValue = strstr(endOfName + 1, ATTRIBUTE_TERMINATOR);
-  if(endOfValue == NULL) {
-    return NULL;
-  }
-
-  valueLen = endOfValue - endOfName - 1;
-  currentValue = (char *)MALLOC(valueLen + 1);
-  if(currentValue != NULL) {
-    memcpy(currentValue, endOfName + 1, valueLen);
-    currentValue[valueLen] = '\0';
-  }
-
-  return(currentValue);
-
-}
-
 /*-------------- struct registrations functions ---------*/
 /* the following functions operates on elements of struct registrations */
 
@@ -800,7 +744,7 @@ NwsAttributeValue(const NwsAttribute ofNwsAttribute) {
  */
 int
 InitRegistrations(registrations **r) {
-	*r = (registrations *) MALLOC(sizeof(registrations));
+	*r = (registrations *) MALLOC(sizeof(registrations), 0);
 	if (*r == NULL) {
 		return 0;
 	}
@@ -824,13 +768,9 @@ CheckRegistrations(registrations *r) {
 	/* do we have extra elements handy? */
 	if (r->dirty <= 0) {
 		/* we don't have extra one: we have to realloc */
-		r->vals = (char **)REALLOC((void *)r->vals, sizeof(char *)*(r->howMany+REG_CHUNK));
+		r->vals = (char **)REALLOC((void *)r->vals, sizeof(char *)*(r->howMany+REG_CHUNK), 1);
 		/* let's make room for the expirations */
-		r->expirations = (unsigned long *)REALLOC((void*)r->expirations, sizeof(long)*(r->howMany+REG_CHUNK));
-		/* check */
-		if (r->vals == NULL || r->expirations == NULL) {
-			ABORT("CheckRegistrations: out of memory\n");
-		}
+		r->expirations = (unsigned long *)REALLOC((void*)r->expirations, sizeof(long)*(r->howMany+REG_CHUNK), 1);
 
 		/* let's zeroes the pointers */
 		memset(&(r->vals[r->howMany]), 0, sizeof(char *)*REG_CHUNK);
@@ -863,10 +803,7 @@ InsertRegistration(	registrations *r,
 	}
 
 	/* we get only the first object (in case there are more then one) */
-	obj = (char *)MALLOC(name - object + OBJECT_TERMINATOR_LEN + 1);
-	if (obj == NULL) {
-		ABORT("InsertRegistration: out of memory\n");
-	}
+	obj = (char *)MALLOC(name - object + OBJECT_TERMINATOR_LEN + 1, 1);
 	memcpy(obj, object, name - object + OBJECT_TERMINATOR_LEN);
 	obj[name - object + OBJECT_TERMINATOR_LEN] = '\0';
 	newName = NwsAttributeValue_r(FindNwsAttribute(obj, "name"));
@@ -989,13 +926,16 @@ SearchForName(	registrations *r,
 
 		/* pick the name of the element */
 		tmp = NwsAttributeValue_r(FindNwsAttribute(r->vals[i], "name"));
-		if (tmp == NULL) {
-			ABORT("SearchForName: out of memory\n");
-		}
 		if (matchExact) {
 			done = strcmp(tmp, name);
 		} else {
-			done = strncmp(tmp, name, len);
+			/* let's see if we match it */
+			if (strmatch(tmp, name)) {
+				/* found it */
+				done = 0;
+			} else {
+				done = strncmp(tmp, name, len);
+			}
 		}
 		FREE(tmp);
 		if (done == 0) {
@@ -1013,13 +953,14 @@ SearchForName(	registrations *r,
 	if (done == 0) {
 		while ((i + 1) < r->howMany) {
 			tmp = NwsAttributeValue_r(FindNwsAttribute(r->vals[i+1], "name"));
-			if (tmp == NULL) {
-				ABORT("SearchForName: out of memory\n");
-			}
 			if (matchExact) {
 				ln =  strcmp(tmp, name);
 			} else {
-				ln =  strncmp(tmp, name, len);
+				if (strmatch(tmp, name)) {
+					ln = 0;
+				} else {
+					ln = strncmp(tmp, name, len);
+				}
 			}
 			FREE(tmp);
 			if (ln == 0) {
@@ -1118,4 +1059,38 @@ SearchForObject(	registrations *r,
 	return ret;
 }
 
+/* utility function to get a tab-separated list of names from a
+ * registrations structure. Returns a char * that needs to be freed */
+char *
+GenerateList(const registrations *r) {
+	char *tmp, *name;
+	int len, i, l;
+
+	/* sanity check */
+	if (r == NULL) { 
+		ERROR("GenerateList: NULL parameter\n");
+		return NULL;
+	}
+
+
+	/* let's generate the list */
+	for (tmp = NULL, len = 0, i = 0;  i < r->howMany; i++) {
+		name = NwsAttributeValue_r(FindNwsAttribute(r->vals[i], "name"));
+		if (name == NULL) {
+			WARN("GenerateList: no name??\n");
+			continue;
+		}
+		l = strlen(name);
+		tmp = REALLOC(tmp, len + l + 2, 1);
+		if (len > 0) {
+			tmp[len] = '\t';
+			len++;
+		}
+		memcpy(tmp + len, name, l + 1);
+		len += l;
+		FREE(name);
+	}
+
+	return tmp;
+}
 
