@@ -1,4 +1,4 @@
-/* $Id: nws_api.h,v 1.83 2004/11/09 00:33:42 graziano Exp $ */
+/* $Id: nws_api.h,v 1.106 2005/04/28 21:52:32 graziano Exp $ */
 
 #ifndef NWS_API_H
 #define NWS_API_H
@@ -324,13 +324,16 @@ NWSAPI_NextObject(NWSAPI_ObjectSet objects,
  * The following functions operates on object and objectSet. They usually
  * returns 1 on success. 
  * AddObject adds an object to an objectSet.
+ * DeleteObject deletes an object from an objectSet
  * NewObjectSet creates an empty objectSet.
  * NWSAPI_FreeObjectSet free the memory used by an objectSet.
  */
 int
 NWSAPI_AddObject(	NWSAPI_ObjectSet *toSet,
 			const NWSAPI_Object obj);
-
+int
+NWSAPI_DeleteObject(	NWSAPI_ObjectSet *fromSet,
+			const NWSAPI_Object obj);
 NWSAPI_ObjectSet
 NWSAPI_NewObjectSet();
 
@@ -353,7 +356,7 @@ NWSAPI_FindNwsAttribute(	const NWSAPI_Object object,
  * #filter#.  Copies the retrieved hosts into #whereTo# and stores the count of
  * retrieved hosts in #numberReturned#.  Use #timeout# seconds as
  * communication timeout (-1 enable automatic timeout, 0 disable
- * timeout). 
+ * timeout). If succesfull #whereTo# needs to be freed by the caller.
  * Returns 1 if successful, else 0.
  */
 int
@@ -412,6 +415,14 @@ NWSAPI_GetObjectsTimeout(	const char *filter,
 				int timeout);
 #define NWSAPI_GetObjects(a, b) NWSAPI_GetObjectsTimeout(a, b, -1)
 
+/**
+ * Small utility functions: returns in <hosts> the comma-separated list
+ * of hostnames member of clique <clique_name> Returns the number of
+ * hostnames in <hosts> or 0 in case of error. <hosts> needs to be freed.
+*/
+int
+NWSAPI_GetCliqueHosts(	const char *clique_name,
+			char **hosts);
 
 /**
  * Add #whichNS# to the list of known nameserver. If #whichNS# is NULL.
@@ -421,6 +432,11 @@ NWSAPI_GetObjectsTimeout(	const char *filter,
 int
 NWSAPI_UseNameServer(const NWSAPI_HostSpec *whichNS);
 
+/**
+ * Set the Proxy to use. It works as UseNameServer but for the proxy.
+ */
+int
+NWSAPI_UseProxy(const NWSAPI_HostSpec *whichProxy);
 
 
 /*
@@ -518,6 +534,16 @@ NWSAPI_GetMeasurements(const char *seriesName,
         NWSAPI_GetMeasurements(which, NWSAPI_BEGINNING_OF_TIME, where, 1, NULL)
 
 
+/**
+ * when query the proxy, you can tell the library either to do a perfect
+ * match on the hostname, or to just match the name. For example, there
+ * are a lot of machines that reports their name without domain name: in
+ * this case foo.bar.edu and foo are considered different host. If you
+ * #relax# is 1, then foo.bar.edu and foo will be considered equivalent.
+ */
+void
+NWSAPI_RelaxHostNameCheck(int relax);
+
 /**
  * Similar to GetMeasurements, but retrieve Forecastes insteades. The
  * parameters are the same as GetMeasuremets expect for #whereTo# that
@@ -535,35 +561,144 @@ NWSAPI_GetForecasts(	const char *seriesName,
 
 
 /**
- * Asks a NWS proxy (specified in #proxy#) for (only) the latest  forecast
- * for #hosts#. #resource# and #opts# specifies which resource (and
- * relative options) to search for. If succesfull returns the # of 
- * forecasts in #col# (otherwise 0): the memory for #col# need to be freed. The
- * number of forecasts returned depends on the resource: for any single
- * host resource (ie availableCPU) are the number of #hosts#, otherwise
- * for 2 hosts resources (ie bandwitdhTcp) square that number.
+ * Asks a NWS proxy (see UseProxy()) for (only) the latest  forecast
+ * for #hosts# (been #hosts# a comma separated list of host:port) .
+ * #resource# and #opts# specifies which resource (and relative options)
+ * to search for. If succesfull returns the # of forecasts in #col#
+ * (otherwise 0): the memory for #col# need to be freed. The number of
+ * forecasts returned depends on the resource: for any single host
+ * resource (ie availableCPU) are the number of #hosts#, otherwise for 2
+ * hosts resources (ie bandwitdhTcp) square that number.
+ *
+ * When #valid# is greater then 0, all data with a timestamp older then
+ * now - #valid# will be discarded as stale.
+ *
+ * #mode# and #level# are positive when virtual data are request: in this
+ * mode, the proxy tries to find super-cliques and substitutes
+ * measurements of these hosts for missing data.
+ *
+ * #mode# can be MODE_MAX (returns the maximum value found in the path),
+ * MODE_MIN (minimum in the path). Any other values will disable the
+ * virtual call, and returns only the real measurements.
+ *
+ * Example: in hosts you had "A,B,C" now you have the following:
+ * 	col[i + j*3] will contain the forecast from host j to host i. So
+ * 	the bandwitdh from B to C will be in col[5] (2 + 1*3). 
+ *
+ * If the proxy doesn't have data for a specific series, the timestamp of
+ * that series will be 0. In the previous example, if there is no data
+ * for B to C, then
+ * 	col[5].measurement.timeStamp 
+ * will be 0.
+ *
+ */
+#define MODE_NO_VIRTUAL (0)
+#define MODE_MAX (1)
+#define MODE_MIN (2)
+int
+NWSAPI_GetVirtualForecasts(	const char *resource,
+				const char *opts,
+				const char *hosts,
+				int mode,
+				int level,
+				unsigned long valid,
+				NWSAPI_ForecastCollection **col,
+				const int timeOut);
+#define NWSAPI_GetAllForecasts(r, o, h, v, c, t) \
+	NWSAPI_GetVirtualForecasts(r, o, h, MODE_NO_VIRTUAL, 0, v, c, t)
+
+/**
+ * This is a small utility functions used by GetVirtualForecasts and
+ * others to compare 2 hosts. With this we try to work around broken DNS
+ * or wrong DNS configurations so that hosts that doesn't report their
+ * domain name can be found anyway (for example node111.whatever.edu
+ * should match node111:8060). #src# and #dst# are the 2 host to compare.
+ * Returns 1 id the hosts are 'similar', 0 otherwise.
  */
 int
-NWSAPI_GetAllForecasts(	NWSAPI_HostSpec *proxy,
-			char *resource,
-			char *opts,
-			char **hosts,
-			NWSAPI_ForecastCollection **col,
-			int timeOut);
+NWSAPI_HostCompare(	const char *src,
+			const char *dst);
+
+
+/**
+ * Similar to GetVirtualForecasts, returns the series of a set of
+ * cliques. The clique names (#cliqueName#) is a comma separated lists of
+ * cliques names, needs to be specified and has to be present in the
+ * Nameserver (that you can set using UseNameServer()). Returns the lists
+ * of hosts in <hosts> as a comma-separated list of names:port (needs to
+ * be freed) and the forecasts in #col#: depending on the resource there
+ * are either the returned number of hosts or the square root.
+ *
+ * When #valid# is greater than 0, data with timestamp older then now -
+ * #valid# will be considered stale and discarded.
+ *
+ * NOTE: despite the name, cliqueName can be any activity, but  only with
+ * clique (large number of hosts) this calls make sense.
+ *
+ * GetCliqueForecasts is just to retrieve the series of only one clique
+ * (no virtualization done at all)
+ */
+int
+NWSAPI_GetCliquesForecasts(	const char *resource,
+				const char *cliqueName,
+				char **hosts,
+				int mode,
+				int level,
+				unsigned long valid,
+				NWSAPI_ForecastCollection **col,
+				int timeOut);
+#define NWSAPI_GetCliqueForecasts(r, c, h, v, col, t) \
+	NWSAPI_GetCliquesForecasts(r, c, h, 0, 0, v, col, t)
 
 /**
  * Asks a NWS #proxy# to start fetching series for #hosts# and
- * #resource#, possibly matching the options in #opts#. hosts has its
- * last element == NULL.
+ * #resource#, possibly matching the options in #opts#. hosts is a
+ * comma-separated list of hosts.
  * Returns 0 on failure, 1 on success.
  */
 int
-NWSAPI_StartFetching(	NWSAPI_HostSpec *proxy,
-			char *resource,
-			char *opts,
-			char **hosts,
+NWSAPI_StartFetching(	const NWSAPI_HostSpec *proxy,
+			const char *resource,
+			const char *opts,
+			const char *hosts,
 			int timeOut);
 
+/**
+ * Similar to NWSAPI_StartFetching, but asks the proxy to start fetching
+ * series for activity <clique_name>. Returns 0 on failure, 1 on success.
+ */
+int
+NWSAPI_StartCliqueFetching(	const NWSAPI_HostSpec *proxy,
+				const char *clique_name,
+				int timeOut);
+
+
+/**
+ * EXPERIMENTAL: finds a path (of hosts and cliques) from host #src# to
+ * host #dst#. #cliques# is a list of cliques used to find the path,
+ * #path# will contains two attribute: hosts and cliques (the hosts hops
+ * and the cliques they belong to) and #level# is how deep to look for
+ * a path. Returns 0 if no path have been found otherwise 1 (and #path#
+ * will contain the way and its memory will need to be freed).
+ */
+int
+NWSAPI_FindHostsPath(	const char *src,
+			const char *dst,
+			NWSAPI_ObjectSet cliques,
+			NWSAPI_ObjectSet *path,
+			int level);
+int
+NWSAPI_FindCliquesPath(	const char *src,
+			const char *dst,
+			NWSAPI_ObjectSet cliques,
+			NWSAPI_ObjectSet *path,
+			int level);
+/* EXPERIMENTAL: don't use it */
+int
+NWSAPI_FindCliquesTree(	char ***cliques,
+			int **tree);
+
+
 /**
  * Contacts the chosen memory (see UseMemory) to store the #howMany#-long
  * series of measurements #measurements# under the name #seriesName#.  Also
@@ -679,8 +814,16 @@ NWSAPI_EnvironmentValue(const char *name, const char *defaultValue);
 #	define GetMeasurements NWSAPI_GetMeasurements
 #	define GetMeasurement NWSAPI_GetMeasurement
 #	define PutMeasurements NWSAPI_PutMeasurements
+#	define RelaxHostNameCheck NWSAPI_RelaxHostNameCheck
+#	define GetVirtualForecasts NWSAPI_GetVirtualForecasts
+#	define HostCompare NWSAPI_HostCompare
 #	define GetAllForecasts NWSAPI_GetAllForecasts
+#	define GetCliquesForecasts NWSAPI_GetCliquesForecasts
+#	define GetCliqueForecasts NWSAPI_GetCliqueForecasts
 #	define StartFetching NWSAPI_StartFetching
+#	define StartCliqueFetching NWSAPI_StartCliqueFetching
+#	define FindHostsPath NWSAPI_FindHostsPath
+#	define FindCliquesPath NWSAPI_FindCliquesPath
 #	define SeriesName NWSAPI_SeriesName
 #	define SeriesName_r NWSAPI_SeriesName_r
 #	define UseMemory NWSAPI_UseMemory
@@ -724,6 +867,7 @@ NWSAPI_EnvironmentValue(const char *name, const char *defaultValue);
 #	define FirstObject NWSAPI_FirstObject
 #	define ForEachObject NWSAPI_ForEachObject
 #	define AddObject NWSAPI_AddObject
+#	define DeleteObject NWSAPI_DeleteObject
 #	define NewObjectSet NWSAPI_NewObjectSet
 #	define FreeObjectSet NWSAPI_FreeObjectSet
 #	define GetHostsTimeout NWSAPI_GetHostsTimeout
@@ -735,6 +879,7 @@ NWSAPI_EnvironmentValue(const char *name, const char *defaultValue);
 #	define GetObjects NWSAPI_GetObjects
 #	define GetObjectsTimeout NWSAPI_GetObjectsTimeout
 #	define UseNameServer NWSAPI_UseNameServer
+#	define UseProxy NWSAPI_UseProxy
 #	define HaltActivity NWSAPI_HaltActivity
 #	define StartActivity NWSAPI_StartActivity
 #	define RestartActivities NWSAPI_RestartActivities
