@@ -1,4 +1,4 @@
-/* $Id: memory.c,v 1.18 2004/10/21 05:49:26 graziano Exp $ */
+/* $Id: memory.c,v 1.19 2005/05/09 20:48:34 graziano Exp $ */
 
 #include "memory.h"
 
@@ -22,7 +22,7 @@ MemoryUseSkill(	const char *options,
 		} else {
 			ret = PassiveMemoryGetFree(&res);
 		}
-		AppendResult(freeMemory, opts, ret, res, length, results);
+		AppendResult(freeMemory, opts, ret, res, 0, length, results);
 	}
 	FREE(tmp);
 }
