@@ -1,4 +1,4 @@
-/* $Id: tcp_bw_exp.c,v 1.107 2004/12/02 04:17:43 graziano Exp $ */
+/* $Id: tcp_bw_exp.c,v 1.110 2005/05/09 20:48:34 graziano Exp $ */
 
 #include "config_nws.h"
 
@@ -439,6 +439,7 @@ TerminateTcpExp(	MessageHeader *header,
 	int returnValue, tmp;
 	struct timeval timeOut;
 	struct sockaddr_in client;
+	char *c;
 	SOCKLEN_T bufferSizeSize = sizeof(bufferSize);
 	SOCKLEN_T clientSize = sizeof(client);
 	Socket clientSock;
@@ -456,7 +457,15 @@ TerminateTcpExp(	MessageHeader *header,
 		return 0;
 	}
 
-	LOG3("Servicing TCP(%d, %d, %d)\n", sizes.expSize, sizes.bufferSize, sizes.msgSize);
+	/* we got here a dotted notation becasue we don't want to go to
+	 * DNS and get stuck there */
+	c = PeerName_r(*sender);
+	if (c != NULL) {
+		LOG4("Servicing TCP(%d, %d, %d) from %s\n", sizes.expSize, sizes.bufferSize, sizes.msgSize, c);
+		FREE(c);
+	} else {
+		LOG3("Servicing TCP(%d, %d, %d)\n", sizes.expSize, sizes.bufferSize, sizes.msgSize);
+	}
 
 	/* The following statement causes problems if we're behind a
 	 * firewall. */
@@ -684,8 +693,8 @@ TcpLtBwUseSkill(	const char *options,
 			sprintf(opts, "buffer:%d\tmessage:%d\tsize:%d\ttarget:%s", sizes.bufferSize / ONE_K, sizes.msgSize / ONE_K, sizes.expSize / ONE_K, HostCImage(&host));
 			ret = InitiateTcpExp(address, host.port, TCP_BW_REQ, &sizes, timeout, &res[0], &childToParent);
 		}
-		AppendResult(bandwidthTcp, opts, ret, res[0], length, results);
-		AppendResult(latencyTcp, opts, ret, res[1], length, results);
+		AppendResult(bandwidthTcp, opts, ret, res[0], 0, length, results);
+		AppendResult(latencyTcp, opts, ret, res[1], 0, length, results);
 	}
 	FREE(tmp);
 }
