@@ -1,4 +1,4 @@
-/* $Id: clique_protocol.c,v 1.217 2004/12/02 04:12:42 graziano Exp $ */
+/* $Id: clique_protocol.c,v 1.227 2005/05/27 00:49:18 graziano Exp $ */
 
 #include "config_nws.h"
 #include <sys/types.h>       /* required for ntohl() on Cray */
@@ -12,7 +12,7 @@
 #include "cliques.h"         /* clique messages and structures */
 #include "diagnostic.h"      /* FAIL() WARN() LOG() */
 #include "dnsutil.h"         /* IP/DNS conversion */
-#include "experiments.h"    /* experiment structure manipulation */
+#include "nws_memory.h"    /* experiment structure manipulation */
 #include "host_protocol.h"   /* host connection protocol */
 #include "messages.h"        /* RegisterListener() */
 #include "osutil.h"          /* CurrentTime() */
@@ -23,8 +23,6 @@
 
 #include "nws_api.h"
 
-#define DEFAULT_CLIQUE_PERIOD "120"
-#define KEEP_A_LONG_TIME 315360000.0
 /* inital factor to multiply period by for clique t/o */
 #define GAIN 0.15
 
@@ -158,10 +156,7 @@ CreateNewHostList(	TokenInfo *info) {
 
 	/* let's make room for the list */
 	if (info->list == NULL) {
-		info->list = (unsigned int *)MALLOC(sizeof(unsigned int) * clique->count);
-		if (info->list == NULL) {
-			ABORT("CreateNewHostList: out of memory\n");
-		}
+		info->list = (unsigned int *)MALLOC(sizeof(unsigned int) * clique->count, 1);
 
 		/* let's initialize the seed */
 		srand((unsigned int)CurrentTime());
@@ -235,10 +230,7 @@ AddNewClique(	Clique *newClique,
 
 	GetNWSLock(&lock);
 	cliquesKnown++;
-	cliqueTokens = REALLOC(cliqueTokens, sizeof(TokenInfo) * cliquesKnown);
-	if (cliqueTokens == NULL) {
-		ABORT("AddNewClique out of memory\n");
-	}
+	cliqueTokens = REALLOC(cliqueTokens, sizeof(TokenInfo) * cliquesKnown, 1);
 	info = &cliqueTokens[cliquesKnown - 1];
 	info->clique = *newClique;
 	info->skill = (KnownSkills) i;
@@ -442,10 +434,7 @@ RegisterClique(TokenInfo *info) {
 	i = 30 + strlen(info->clique.options);	/* size of the string
 						   (let's be generous:
 						   we'll realloc anyway) */
-	opts = (char *)MALLOC(sizeof(char) * i);
-	if (opts == NULL) {
-		ABORT("RegisterClique: out of memory\n");
-	}
+	opts = (char *)MALLOC(sizeof(char) * i, 1);
 	if (info->clique.options[0] == '\0') {
 		sprintf(opts, "period:%d\tmember:", (int)info->clique.period);
 	} else {
@@ -468,10 +457,7 @@ RegisterClique(TokenInfo *info) {
 		Host2Cookie(tmp, info->clique.members[i].port, &member);
 		FREE(tmp);
 		vstrncpy(s, sizeof(s), 2, (i > 0) ? "," : "", HostCImage(&member));
-		opts = (char *) REALLOC(opts, strlen(opts) + strlen(s) + 1);
-		if (opts == NULL) {
-			ABORT("RegisterClique: out of memory\n");
-		}
+		opts = (char *) REALLOC(opts, strlen(opts) + strlen(s) + 1, 1);
 		strcat(opts, s);
 	}
 
@@ -806,7 +792,7 @@ HandleCliqueMessage( 	Socket *sd,
 		} else {
 			/* series registration will be here */
 			descriptor.repetitions = (int)d[2];
-			data = (char *)MALLOC(descriptor.repetitions + 1);
+			data = (char *)MALLOC(descriptor.repetitions + 1, 0);
 			if(data == NULL) {
 				ERROR("HandleCliqueMessage: out of memory\n");
 				DROP_SOCKET(sd);
@@ -833,7 +819,7 @@ HandleCliqueMessage( 	Socket *sd,
 	case CLIQUE_NEW_MEMBER:
 	case CLIQUE_REMOVE_MEMBER:
 		descriptor.repetitions = header.dataSize;
-		data = (char *)MALLOC(header.dataSize + 1);
+		data = (char *)MALLOC(header.dataSize + 1, 0);
 		if (data == NULL) {
         		ERROR("HandleCliqueMessage: receive failed\n");
 			DROP_SOCKET(sd);
@@ -891,7 +877,7 @@ CliqueMembership(	struct host_cookie *cookie,
 	}
 
 	/* prepare the data to send */
-	data = (char *)MALLOC(strlen(cliqueName) + strlen(hostName) + 1);
+	data = (char *)MALLOC(strlen(cliqueName) + strlen(hostName) + 3, 0);
 	if (data == NULL) {
 		ERROR("CliqueMembership: out of memory\n");
 		return 0;
@@ -1020,14 +1006,14 @@ RunCliqueExperiment(	TokenInfo *info) {
 	IPAddress addr;
 	double d[3];
 
+	clique = &info->clique;			/* alias */
+
 	/* we don't do experiment with ourselves */
 	if (info->list[info->nextHost] == info->myIndex) {
 		WARN1("RunCliqueExperiment: clique %s called with my index\n", clique->name);
 		return 0;
 	} 
 
-	clique = &info->clique;			/* alias */
-
 	/* let's build the option with the member we want to do
 	 * experiments with */
 	options = strdup(clique->options);
@@ -1070,7 +1056,7 @@ RunCliqueExperiment(	TokenInfo *info) {
 	}
 
 	/* allocate space for target:name:port */
-	options = (char *)REALLOC(options, strlen(options) + strlen(tmp) + 14);
+	options = (char *)REALLOC(options, strlen(options) + strlen(tmp) + 14, 0);
 	if (options == NULL) {
 		ERROR("RunCliqueExperiment: out of memory\n");
 		FREE(tmp);
@@ -1085,7 +1071,7 @@ RunCliqueExperiment(	TokenInfo *info) {
 	/* and now add the fork options if not specified */
 	tmp = GetOptionValue(options, "fork", NULL);
 	if (tmp == NULL) {
-		options = (char *)REALLOC(options, strlen(options) + 10);
+		options = (char *)REALLOC(options, strlen(options) + 10, 0);
 		if (options == NULL) {
 			ERROR("RunCliqueExperiment: out of memory\n");
 			return 0;
@@ -1108,7 +1094,7 @@ RunCliqueExperiment(	TokenInfo *info) {
 		}
 		/* Child process. */
 		/* now add the socket for the child to parent communication */
-		options = (char *)REALLOC(options, strlen(options) + 20);
+		options = (char *)REALLOC(options, strlen(options) + 20, 0);
 		if (options == NULL) {
 			ERROR("RunCliqueExperiment: out of memory\n");
 			return 0;
@@ -1120,12 +1106,12 @@ RunCliqueExperiment(	TokenInfo *info) {
 	timeOut = (unsigned int)(clique->period) - 1;
 	UseSkill(info->skill, options, timeOut, &results, &resultsLength);
 	FREE(options);
-	toStore.timeStamp = CurrentTime();
 
 	ret = 0;
 	for(i = 0; i < resultsLength; i++) {
 		if(results[i].succeeded) {
 			toStore.measurement = results[i].measurement;
+			toStore.timeStamp = results[i].timeStamp;
 			toRegister = CreateSeriesObject(NULL, 
 					EstablishedRegistration(),
 					ResourceLabel(results[i].resource),
@@ -1205,6 +1191,14 @@ RunCliqueJob(TokenInfo *info) {
 	now = CurrentTime();
 	clique = &info->clique;
 
+	/* it could be that the clique is of size 1, in this case there
+	 * is nothing to do */
+	if (clique->count < 2) {
+		LOG1("RunCliqueJob: clique %s of size 1, nothing to do\n", clique->name);
+		info->nextRunTime = now + clique->period;
+		return;
+	}
+
 	/* we calculate here the distance from leader */
 	if (info->myIndex > clique->leader) {
 		indx = info->myIndex - clique->leader;
@@ -1343,14 +1337,14 @@ RunCliqueJob(TokenInfo *info) {
 	if (info->nextHost == 0) {
 		/* let's record the time of this measurement: we record
 		 * only the first host we try. We give a few s slack to
-		 * account for across the second measurment on different
+		 * account for across the second measurement on different
 		 * host. */
 		if (info->lastMeasurement + clique->period > now + 1){
 			WARN1("RunCliqueJob %s is early\n", clique->name);
 
 			/* let's punt to the right time */
 			info->nextRunTime = info->lastMeasurement + clique->period + 1;
-			return ;
+			return;
 		}
 		info->lastMeasurement = now;
 	}
@@ -1532,7 +1526,7 @@ StartCliqueActivity(const char *registration,
 	clique.instance = 1;
 	SAFESTRCPY(clique.skill, skillName);
 	SkillOptions(skill, options, clique.options, sizeof(clique.options));
-	tmp = GetOptionValue(options, "period", DEFAULT_CLIQUE_PERIOD);
+	tmp = GetOptionValue(options, "period", SkillOptionDefault(skill, "period"));
 	clique.period = strtod(tmp, NULL);
 	FREE(tmp);
 	clique.timeOut = 3 * clique.period;
@@ -1560,11 +1554,6 @@ StartCliqueActivity(const char *registration,
 	}
 	FREE(tmp);
 
-	/* let's check that we have at least 2 members here */
-	if (clique.count < 2) {
-		FAIL1("StartCliqueActivity: not enough members for %s\n", clique.name);
-	}
-
 	/* now let's try to add this clique to the known cliques */
 	i = AddNewClique(&clique, &tokenIndex);
 	if (i == -1) {
@@ -1572,6 +1561,11 @@ StartCliqueActivity(const char *registration,
 		return 0;
 	}
 
+	/* let's check that we have at least 2 members here */
+	if (clique.count < 1) {
+		WARN1("StartCliqueActivity: just one member for %s\n", clique.name);
+	}
+
 	GetNWSLock(&lock);
 	info = &cliqueTokens[tokenIndex];
 
@@ -1584,6 +1578,15 @@ StartCliqueActivity(const char *registration,
 			return 0;
 		}
 
+		/* fix the clique status */
+		info->state = TOKEN_HELD;
+		info->cycleStartTime = 0;
+		info->nextRunTime = 0.0;
+		info->lastMeasurement = 0.0;
+
+		/* let's reset the new sequence */
+		CreateNewHostList(info);
+
 		/* update the clique */
 		info->clique = clique;
 	}
@@ -1622,7 +1625,7 @@ StopCliqueActivity(const char *reg) {
 	}
 
 	if (!FindTokenInfo(reg, &tokenIndex)) {
-		FAIL1("StopCliqueActivity: unknown clique %s", reg);
+		FAIL1("StopCliqueActivity: unknown clique %s\n", reg);
 	}
 
 	/* kinda sucks: we need to look across a send */
