@@ -1,11 +1,11 @@
-/* $Id: skills.c,v 1.84 2004/11/23 19:45:13 graziano Exp $ */
+/* $Id: skills.c,v 1.96 2005/07/20 19:27:23 graziano Exp $ */
 
 #include "config_nws.h"
 
 #include <stdlib.h>
 
 #include "diagnostic.h"
-#include "experiments.h"
+#include "nws_memory.h"
 #include "host_protocol.h"
 #include "skills.h"
 #include "osutil.h"
@@ -18,8 +18,8 @@
 #include "tcp_bw_exp.h"
 #include "fstests.h"
 #include "availtest.h"
-#include "starttest.h"
 #include "memorySpeed.h"
+#include "batchtest.h"
 
 static int wouldYouFork = 1;
 static void *lock = NULL;			/* local lock */
@@ -27,7 +27,7 @@ static void *lock = NULL;			/* local lock */
 
 
 /* these are all the options we understand */
-static const char* OPTION_FORMATS = "buffer:0_to_1_int\tmessage:0_to_1_int\tnice:0_to_10_int\tpath:1_to_10_string\ttype:0_to_1_string\tsize:0_to_1_int\ttarget:1_to_100_sensors\tfstype:0_to_1_string\tfsblock:0_to_1_int\tfssize:0_to_1_int\tfsbufmode:0_to_1_int\tfstmpdir:0_to_1_string\tfsphysmem:0_to_1_int\tmode:0_to_1_string\tloops:0_to_1_int";
+static const char* OPTION_FORMATS = "buffer:0_to_1_int\tmessage:0_to_1_int\tnice:0_to_10_int\tpath:1_to_10_string\ttype:0_to_1_string\tsize:0_to_1_int\ttarget:1_to_100_sensors\tfstype:0_to_1_string\tfsblock:0_to_1_int\tfssize:0_to_1_int\tfsbufmode:0_to_1_int\tfstmpdir:0_to_1_string\tfsphysmem:0_to_1_int\tmode:0_to_1_string\tloops:0_to_1_int\tqueues:0_to_10_string\tnoderanges:0_to_128_string\tdbpath:0_to_1_string";
 
 
 /*
@@ -53,7 +53,11 @@ static const NwsResource myResources[RESOURCE_COUNT] = {
 	{"Megabytes/second", "fsreadSpeed", fsreadSpeed},
 	{"Megabytes/second", "fswriteSpeed", fswriteSpeed},
 	{"Seconds", "upTime", upTime},
-	{"Seconds", "startTime", startTime},
+	{"Seconds", "startTime", upTime},
+#ifdef EXPERIMENTAL
+	{"Seconds", "batchQueue", batchQueue},
+	{"Seconds", "batchQueueLog", batchQueueLog},
+#endif
 	{"Megabytes/second", "memorySpeed", memorySpeed}
 };
 
@@ -108,17 +112,20 @@ typedef void (*MonitorUseSkill)(const char *options, int *length, SkillResult **
  * 		snprintf(opts, sizeof(opts), "pippo:%s,target:%s",
  * 				myCommonOpts, option);
  * 		if (myExperiment(...,&res)) {
- * 			AppendResult(skill, opts, 1, res, lenght,results);
+ * 			AppendResult(skill, opts, 1, res, length,results);
  * 		} else {
- * 			AppendResult(skill, opts, 0, 0, lenght,results);
+ * 			AppendResult(skill, opts, 0, 0, length,results);
  * 		}
  * 	}
  *	free(myCommonOpts);
  *	free(targets);
  *
- *
+ * defaultPeriod is the default periodicity for the particular skill.
+ * It's used whenever the user doens't specify a period.
  *
  */
+#define DEAFULT_PERIODIC_PERIOD 10
+#define DEAFULT_CLIQUE_PERIOD 120
 #define MAX_RESOURCES_PER_SKILL 2
 typedef struct {
 	const char *name;
@@ -129,18 +136,21 @@ typedef struct {
 	const int resourcesCount;
 	MonitorIsAvailable skillIsAvailable;
 	MonitorUseSkill useSkill;
+	int defaultPeriod;
 } NwsSkill;
 
 static const NwsSkill mySkills[SKILL_COUNT] = {
-	{"cpuMonitor", PERIODIC_CONTROL, "nice", "0", {availableCpu, currentCpu}, 2, &HybridCpuMonitorAvailable, &HybridCpuUseSkill},
-	{"diskMonitor", PERIODIC_CONTROL, "path", NULL, {freeDisk}, 1, &DiskFreeAvailable, &DiskFreeUseSkill}, 
-	{"memoryMonitor", PERIODIC_CONTROL, "type", "passive", {freeMemory}, 1, &MemoryMonitorAvailable, &MemoryUseSkill},
-	{"tcpConnectMonitor", CLIQUE_CONTROL, "", NULL, {connectTimeTcp}, 1, &TcpConnectAvailable, &TcpConnectUseSkill}, 
-	{"tcpMessageMonitor", CLIQUE_CONTROL, "buffer,message,size", "32,16,256", {bandwidthTcp, latencyTcp}, 2, &TcpLtBwAvailable, &TcpLtBwUseSkill},
-	{"filesystemMonitor", PERIODIC_CONTROL, "fstype,fsblock,fssize,path,fsbufmode,fstmpdir,fsphysmem", NULL, {fsreadSpeed, fswriteSpeed}, 2, &FileSystemMonitorAvailable, &FileSystemUseSkill},
-	{"availabilityMonitor", PERIODIC_CONTROL, "", NULL, {upTime}, 1, &AvailabilityMonitorAvailable, &AvailabilityUseSkill},
-	{"startMonitor", PERIODIC_CONTROL, "", NULL, {startTime}, 1, &StartAvailable, &StartUseSkill},
-	{"memorySpeedMonitor", PERIODIC_CONTROL, "size,mode,loops", "8192,random,1", {memorySpeed}, 1, &MemorySpeedAvailable, &MemorySpeedUseSkill}
+	{"cpuMonitor", PERIODIC_CONTROL, "nice", "0", {availableCpu, currentCpu}, 2, &HybridCpuMonitorAvailable, &HybridCpuUseSkill, DEAFULT_PERIODIC_PERIOD},
+	{"diskMonitor", PERIODIC_CONTROL, "path", NULL, {freeDisk}, 1, &DiskFreeAvailable, &DiskFreeUseSkill, DEAFULT_PERIODIC_PERIOD}, 
+	{"memoryMonitor", PERIODIC_CONTROL, "type", "passive", {freeMemory}, 1, &MemoryMonitorAvailable, &MemoryUseSkill, DEAFULT_PERIODIC_PERIOD},
+	{"tcpConnectMonitor", CLIQUE_CONTROL, "", NULL, {connectTimeTcp}, 1, &TcpConnectAvailable, &TcpConnectUseSkill, DEAFULT_CLIQUE_PERIOD}, 
+	{"tcpMessageMonitor", CLIQUE_CONTROL, "buffer,message,size", "32,16,256", {bandwidthTcp, latencyTcp}, 2, &TcpLtBwAvailable, &TcpLtBwUseSkill, DEAFULT_CLIQUE_PERIOD},
+	{"filesystemMonitor", PERIODIC_CONTROL, "fstype,fsblock,fssize,path,fsbufmode,fstmpdir,fsphysmem", NULL, {fsreadSpeed, fswriteSpeed}, 2, &FileSystemMonitorAvailable, &FileSystemUseSkill, DEAFULT_PERIODIC_PERIOD},
+	{"availabilityMonitor", PERIODIC_CONTROL, "", NULL, {upTime, startTime}, 2, &AvailabilityMonitorAvailable, &AvailabilityUseSkill, (DEAFULT_PERIODIC_PERIOD * 3)},
+#ifdef EXPERIMENTAL
+	{"batchQueueMonitor", PERIODIC_CONTROL, "queues,noderanges,dbpath", "ALL,ALL,/tmp", {batchQueue,batchQueueLog}, 2, &batchQueueMonitorAvailable, &batchQueueUseSkill, (DEAFULT_PERIODIC_PERIOD * 3)},
+#endif
+	{"memorySpeedMonitor", PERIODIC_CONTROL, "size,mode,loops", "8192,random,1", {memorySpeed}, 1, &MemorySpeedAvailable, &MemorySpeedUseSkill, (DEAFULT_PERIODIC_PERIOD * 12)}
 };
 
 
@@ -150,6 +160,7 @@ AppendResult(	MeasuredResources resource,
 		const char *options,
 		int succeeded,
 		double measurement,
+		double timeStamp,
 		int *howMany,
 		SkillResult **results) {
 	SkillResult *tmp;
@@ -171,21 +182,25 @@ AppendResult(	MeasuredResources resource,
 		tmp = *results;
 	}
 	/* make room for the pointer */
-	tmp = REALLOC(tmp, (*howMany + 1) * sizeof(SkillResult));
-	if (results == NULL) {
-		ABORT("AppendResult: out of memory\n");
-	}
+	tmp = REALLOC(tmp, (*howMany + 1) * sizeof(SkillResult), 1);
 	tmp[*howMany].resource = resource;
 	tmp[*howMany].options = strdup(options);
 	if (tmp[*howMany].options == NULL) {
 		ABORT("AppendResult: out of memory\n");
 	}
 	tmp[*howMany].succeeded = succeeded;
+	/* set the measurement */
 	if (succeeded) {
 		tmp[*howMany].measurement = measurement;
 	} else {
 		tmp[*howMany].measurement = 0.0;
 	}
+	/* and the timestamp */
+	if (timeStamp > 0) {
+		tmp[*howMany].timeStamp = timeStamp;
+	} else {
+		tmp[*howMany].timeStamp = CurrentTime();
+	}
 
 	/* one more result */
 	*howMany = *howMany + 1;
@@ -197,15 +212,13 @@ void
 FreeSkillResults(	int howMany,
 			SkillResult **results) {
 	int i;
-	SkillResult *tmp;
 	
-	tmp = *results;
-
 	for(i = 0; i < howMany; i++) {
-		free(tmp[i].options);
+		free((*results)[i].options);
+	}
+	if (howMany > 0) {
+		FREE(*results);
 	}
-	FREE(tmp);
-	*results = NULL;
 }
 
 
@@ -244,7 +257,7 @@ HandleSkillMessage(Socket *sd,
 
 		/* get the options */
 		descriptor.repetitions = header.dataSize;
-		tmp = (char *)MALLOC(descriptor.repetitions + 1);
+		tmp = (char *)MALLOC(descriptor.repetitions + 1, 0);
 		if (tmp == NULL) {
 			ERROR("HandleSkillMessage: out of memory\n");
 			break;
@@ -355,7 +368,7 @@ GetOptionValue(const char *options,
 
 	/* Merge all #name# option values into a single, comma-delimited
 	 * list. */
-	returnValue = (char *)MALLOC(valuesLen);
+	returnValue = (char *)MALLOC(valuesLen, 1);
 	memset(returnValue, 0, valuesLen);
 	for(c = strstr(options, name); c != NULL; c = strstr(c + 1, name)) {
 		if ((*(c + nameLen) != ':') || ((c != options) && (*(c - 1) != '\t')))
@@ -452,6 +465,7 @@ SkillOptionDefault(	const KnownSkills skill,
 	/* let's find if we know the option for the skill, and wich
 	 * position it has */
 	ret = NULL;
+	value[0] = '\0';
 	v = mySkills[skill].optionsDefault;
 	c = mySkills[skill].options;
 	while (GETTOK(name, c, ",", &c) && GETTOK(value, v, ",", &v)) {
@@ -463,6 +477,17 @@ SkillOptionDefault(	const KnownSkills skill,
 			break;
 		}
 	}
+	
+	/* we don't have period as option, because we treated it
+	 * differently. We return here the default period if period is
+	 * the option */
+	if (ret == NULL && strncmp(option, "period", strlen(option)) == 0) {
+		sprintf(value, "%d", mySkills[skill].defaultPeriod);
+		ret = strdup(value);
+		if (ret == NULL) {
+			ERROR("SkillOptionDefault: out of memory\n");
+		}
+	}
 
 	return ret;
 }
@@ -605,7 +630,9 @@ UseSkill(	KnownSkills skill,
 		*n;
 	int i;
 		
-	*length = 0;			/* not yet data */
+	/* not yet data */
+	*length = 0;
+	*results = NULL;
 
 	/* let's check if the skill is valid */
 	if (skill < 0 || skill > SKILL_COUNT) {
@@ -726,7 +753,7 @@ RemoteUseSkill(	struct host_cookie *cookie,
 	}
 
 	/* makes room for the resources name */
-	*resources = (char **)MALLOC(sizeof(char*) * len);
+	*resources = (char **)MALLOC(sizeof(char*) * len, 0);
 	if (*resources == NULL) {
 		ERROR("RemoteUseSkill: out of memory\n");
 		return 0;
@@ -740,7 +767,7 @@ RemoteUseSkill(	struct host_cookie *cookie,
 		/* we receive the descriptions first: the length is
 		 * dataSize - the size of a double ... */
 		objDes.repetitions = dataSize - HomogenousDataSize(DOUBLE_TYPE, 1, NETWORK_FORMAT);
-		(*resources)[i] = (char *)MALLOC(objDes.repetitions + 1);
+		(*resources)[i] = (char *)MALLOC(objDes.repetitions + 1, 0);
 		if ((*resources)[i] == NULL) {
 			ERROR("RemoteUseSkill: out of memory\n");
 			return 0;
