@@ -800,7 +800,7 @@ nwsMeas2entry(Backend *be, nwsSeriesCache *nser, char *rdn) {
 	val.bv_len = strlen(buf);
 	attr_merge(e, ad_timeStamp, vals);
 
-	sprintf(buf, "%f", nser->exper[0].value);
+	sprintf(buf, "%f", nser->exper[0].measurement);
 	val.bv_val = buf; 
 	val.bv_len = strlen(buf); 
 	attr_merge(e, ad_value, vals);
@@ -897,7 +897,7 @@ nwsForc2entry(Backend *be, nwsSeriesCache *nser, char *rdn) {
 	val.bv_len = strlen(buf);
 	attr_merge(e, ad_timeStamp, vals);
 
-	sprintf(buf, "%f", nser->exper[0].value);
+	sprintf(buf, "%f", nser->exper[0].measurement);
 	val.bv_val = buf; 
 	val.bv_len = strlen(buf); 
 	attr_merge(e, ad_value, vals);
@@ -994,7 +994,7 @@ nwsColl2entry(Backend *be, nwsSeriesCache2Coll *nser_c2c,
 	val.bv_len = strlen(buf);
 	attr_merge(e, ad_timeStamp, vals);
 
-	sprintf(buf, "%f", nser->exper[0].value);
+	sprintf(buf, "%f", nser->exper[0].measurement);
 	val.bv_val = buf; 
 	val.bv_len = strlen(buf); 
 	attr_merge(e, ad_value, vals);
