@@ -1,4 +1,4 @@
-/* $Id: nws_memory.c,v 1.176 2004/11/05 21:40:02 graziano Exp $ */
+/* $Id: nws_memory.c,v 1.195 2005/08/04 00:18:32 graziano Exp $ */
 
 #include "config_nws.h"
 
@@ -19,14 +19,13 @@
 #include "nws_memory.h"
 #include "skills.h"
 #include "nws_db.h"
+#include "nws_daemon.h"
 
 /*
  * This program implements the NWS memory host.  See nws_memory.h for supported
  * messages.
  */
 
-#define KEEP_A_LONG_TIME 315360000.0
-
 /* we keep a list of series we know about, so that we can register them
  * with the nameserver. We have 2: one for the orphaned series and one
  * for the registered series: the orphaned will disappears when the new
@@ -111,7 +110,7 @@ CheckOldNames(	char *names,
 
 	for (where = names + strlen(names); where > names; *(--where) = '\0') {
 		/* look for the beginning of the filename */
-		while (*where != '\n' && where > names) {
+		while (where >= names &&  *where != '\n') {
 			where--;
 		}
 		where++;
@@ -303,36 +302,14 @@ RegisterSeries(	registrations *r,
 	return  (j < r->howMany);
 }
 
-/* 
- * This is somewhat hopeless, but we tried to unregister all the series!!
- */
-static void
-UnregisterSeries(	registrations *r) {
-	int j;
-	char *name, filter[255];
-	const char *myName;
-	
-	myName = EstablishedRegistration();
-	for (j = 0; j < r->howMany; j++) {
-		/* let's get the name and unregister the beast */
-		name = NwsAttributeValue_r(FindNwsAttribute(r->vals[j], "name"));
-		if (name == NULL) {
-			WARN("UnregisterSeries: object with no name??\n");
-			continue;
-		}
-		snprintf(filter, 255, "%s)(memory=%s", name, myName);
-		FREE(name);
-
-		UnregisterObject(filter);
-	}
-
-	return;
-}
 /* todo list when we exit */
 static int
 MyExit(void) {
-	LOG("MyExit: unregistering series, it may take sometime!\n");
-	UnregisterSeries(mySeries);
+	char filter[255];
+
+	LOG("MyExit: unregistering series.\n");
+	snprintf(filter, 255, "(memory=%s)", EstablishedRegistration());
+	UnregisterFilter(filter);
 
 	return 1;
 }
@@ -379,8 +356,6 @@ StoreSeriesName(char *registration) {
 
 	/* let's look for the registration in the good list */
 	if (!SearchForObject(mySeries, obj, &ind)) {
-		/* expiration is bogus so that we'll register the series
-		 * right away */
 		InsertRegistration(mySeries, obj, 10, ind);
 	}
 	FREE(obj);
@@ -388,37 +363,152 @@ StoreSeriesName(char *registration) {
 	return 1;
 }
 
+/* do the real transfer of a series. The string we get a tab-separated
+ * list of series, preceded by the memory (in host:port format). 
+ */
+static int
+DoTransfer(	const char *contents) {
+	char series[255];
+	struct host_cookie mem;
+	int i;
+	size_t j;
+	NWSAPI_Measurement *meas;
+
+	/* sanoty check */
+	if (contents == NULL || strlen(contents) < 5) {
+		ERROR("DoTransfer: invalid parameter\n");
+		return 0;
+	}
+
+	/* the first element is the memory */
+	if (!GETWORD(series, contents, &contents)) {
+		ERROR("DoTransfer: error reading remote memory info\n");
+		return 0;
+	}
+	Host2Cookie(series, DefaultHostPort(MEMORY_HOST), &mem);
+	if (!ConnectToHost(&mem, NULL)) {
+		ERROR("DoTransfer: failed to connect to remote memory\n");
+		return 0;
+	}
+
+	/* let's allocate the space for the measurements */
+	meas = (NWSAPI_Measurement *)MALLOC(fileSize * sizeof(NWSAPI_Measurement), 1);
+
+	/* now let's go though all the series we might have */
+	while(GETWORD(series, contents, &contents)) {
+		/* let's see if we already have this series */
+		if (SearchForName(mySeries, series, 1, &i)) {
+			WARN1("DoTransfer: series already present (%s)\n", series);
+			continue;
+		}
+
+		/* get the experiments from the memory */
+		if (!LoadExperiments(&mem, series, meas, fileSize, 1, &j, NULL, -1)) {
+			WARN1("DoTransfer: couldn't get old data for %s\n", series);
+			continue;
+		}
+
+		/* let's go thorugh the measurments */
+		for (i = 0; i < j; i++) {
+
+		}
+	}
+	FREE(meas);
+
+	return 1;
+}
+
 /*
  * A "local" function of ProcessRequest().  Implements the MEMORY_CLEAN service
  * by deleting all files in the memory directory that have not been accessed
  * within the past #idle# seconds.  Returns 1 if successful, else 0.
  */
 static int
-DoClean(unsigned long idle) {
-	int ret;
+DoClean(	const char *series,
+		unsigned long idle) {
+	int ret, i;
+	char tmp[255], *s, *t;
+	const char *ptr;
 
-	switch (backType) {
-	case LOCALDIR:
-		ret = CleanLocalStates(idle);
-		break;
-
-	case POSTGRES:
-		INFO("DoClean: no cleaning implemented when using database\n");
-		ret = 0;
-		break;
-	
-	case NETLOGGER:
-		INFO("DoClean: no cleaning implemented when using netlooger\n");
-		ret = 0;
-		break;
+	/* if series is empty, we have to operate on all the series we
+	 * are responsible */
+	if (series == NULL || strlen(series) <= 2) {
+		s = GenerateList(mySeries);
+		if (s) {
+			i = strlen(s);
+		} else {
+			i = 0;
+		}
+		t = GenerateList(orphanedSeries);
+		if (t) {
+			REALLOC(s, i + strlen(t) + 2, 1);
+			if (i > 0) { 
+				strcat(s, "\t");
+				memmove(s + i + 1, t, strlen(t) + 1);
+			}
+			FREE(t);
+		}
+	} else {
+		s = strdup(series);
+		if (s == NULL) {
+			ERROR("DoClean: out of memory\n");
+			return 0;
+		}
 	}
 
+	/* let's go through all the series and clean them up */
+	for (ret = 0, ptr = s; GETWORD(tmp, ptr, &ptr);) {
+		if (strlen(tmp) <= 1) {
+			continue;
+		}
+
+		/* now let's check that we are indeed responsible of this
+		 * series: we don't want to delete random files */
+		if (!SearchForName(orphanedSeries, tmp, 1, &i) &&
+				!SearchForName(mySeries, tmp, 1, &i)) {
+			WARN("DoClean: unknown series, not cleaning\n");
+			continue;
+		}
+
+		i = 0;
+		switch (backType) {
+		case LOCALDIR:
+			i = CleanLocalStates(tmp, idle);
+			break;
+
+		case POSTGRES:
+			INFO("DoClean: no cleaning implemented when using database\n");
+			break;
+	
+		case NETLOGGER:
+			INFO("DoClean: no cleaning implemented when using netlooger\n");
+			break;
+		}
+
+		/* if we deleted a series, we need to remove the
+		 * corresponding registration too */
+		if (i) {
+			LOG1("DoClean: removed series %s\n", tmp);
+			if (SearchForName(orphanedSeries, tmp, 1, &i)) {
+				DeleteRegistration(orphanedSeries, i);
+				UnregisterObject(tmp);
+			}
+			if (SearchForName(mySeries, tmp, 1, &i)) {
+				DeleteRegistration(mySeries, i);
+				UnregisterObject(tmp);
+			}
+			ret = 1;
+		}
+
+	}
+	FREE(s);
+
 	return ret;
 }
 
 
 /*
- * Removes from the #auotFetchInfo# module variable all information for the
+ * Removes from the #autoFetchInfo# module variable all information for the
  * client connected to #sock#.
  */
 static void
@@ -589,7 +679,7 @@ AddAutoFetcher(	Socket *sd,
 
 	/* let's make room for the new autofetcher */
 	if (j == -1) {
-		expandedAutoFetches = REALLOC(autoFetches, (i+1)*sizeof(AutoFetchInfo));
+		expandedAutoFetches = REALLOC(autoFetches, (i+1)*sizeof(AutoFetchInfo), 0);
 		if (expandedAutoFetches == NULL) {
 			ERROR("AddAutoFetcher: out of memory\n");
 			return 0;
@@ -708,7 +798,7 @@ ProcessRequest(	Socket *sd,
 			ERROR("ProcessRequest: state receive failed\n");
 		} else {
 			/* we are cool */
-			contents = (char *)MALLOC(stateDesc.rec_count * MAX_RECORD_SIZE);
+			contents = (char *)MALLOC(stateDesc.rec_count * MAX_RECORD_SIZE, 0);
 			if(contents == NULL) {
 				(void)SendMessage(*sd, MEMORY_FAILED, -1);
 				ERROR("ProcessRequest: out of memory\n");
@@ -766,7 +856,7 @@ ProcessRequest(	Socket *sd,
 		} else {
 			/* we are cool */
 			contentsDes.repetitions = stateDesc.rec_size * stateDesc.rec_count;
-			contents = (char *)MALLOC(contentsDes.repetitions + 1);
+			contents = (char *)MALLOC(contentsDes.repetitions + 1, 0);
 			if(contents == NULL) {
 				(void)SendMessage(*sd, MEMORY_FAILED, -1);
 				ERROR("ProcessRequest: out of memory\n");
@@ -821,8 +911,8 @@ ProcessRequest(	Socket *sd,
 		} else {
 			/* we are cool */
 			contentsDes.repetitions = newStateDesc.rec_size * newStateDesc.rec_count + newStateDesc.id_len;
-			contents = (char *)MALLOC(contentsDes.repetitions + 1);
-			tmp = (char *)MALLOC(newStateDesc.id_len + 1);
+			contents = (char *)MALLOC(contentsDes.repetitions + 1, 0);
+			tmp = (char *)MALLOC(newStateDesc.id_len + 1, 0);
 			if(contents == NULL || tmp == NULL) {
 				(void)SendMessage(*sd, MEMORY_FAILED, -1);
 				ERROR("ProcessRequest: out of memory\n");
@@ -852,8 +942,8 @@ ProcessRequest(	Socket *sd,
 	
 			/* here only the name is needed */
 			stateNames = NwsAttributeValue_r(FindNwsAttribute(tmp, "name"));
-			if (stateNames == NULL) {
-				ERROR("ProcessRequest: series without a name?!\n");
+			if (stateNames == NULL || stateNames[0] == '\0') {
+				ERROR("ProcessRequest: series with no name\n");
 				DROP_SOCKET(sd);
 				FREE(tmp);
 				FREE(contents);
@@ -894,7 +984,7 @@ ProcessRequest(	Socket *sd,
 		autoFetchTime -= MicroTime();
 
 		stateNamesDescriptor.repetitions = header.dataSize;
-		stateNames = (char *)MALLOC(header.dataSize);
+		stateNames = (char *)MALLOC(header.dataSize, 0);
 		if(stateNames == NULL) {
 			(void)SendMessage(*sd, MEMORY_FAILED, -1);
 			DROP_SOCKET(sd);
@@ -928,13 +1018,40 @@ ProcessRequest(	Socket *sd,
 		break;
 
 	case MEMORY_CLEAN:
+		/* let's get the idle timeout first */
 		if (!RecvData(*sd, &expiration, &expDescriptor, 1, -1)) {
 			DROP_SOCKET(sd);
-			ERROR("ProcessRequest: data receive failed\n");
-		} else {
-			(void)SendMessage(*sd, MEMORY_CLEANED, -1);
-			(void)DoClean(expiration);
+			ERROR("ProcessRequest: data (idle) receive failed\n");
+			break;
 		}
+
+		/* then the series */
+		contentsDes.repetitions = header.dataSize - HomogenousDataSize(UNSIGNED_LONG_TYPE, 1, NETWORK_FORMAT);
+		contents = MALLOC(contentsDes.repetitions + 1, 1);
+		if (!RecvData(*sd, contents, &contentsDes, 1, -1)) {
+			DROP_SOCKET(sd);
+			ERROR("ProcessRequest: data (series) receive failed\n");
+		} else {
+			contents[contentsDes.repetitions] = '\0';
+			(void)SendMessage(*sd, MEMORY_CLEANED, -1);
+			(void)DoClean(contents, expiration);
+		}
+		FREE(contents);
+
+		break;
+
+	case TRANSFER_SERIES:
+		/*  we are taking over this series from another memory */
+		contentsDes.repetitions = header.dataSize;
+		contents = MALLOC(contentsDes.repetitions + 1, 1);
+		if (!RecvData(*sd, contents, &contentsDes, 1, -1)) {
+			DROP_SOCKET(sd);
+			ERROR("ProcessRequest: data (series) receive failed\n");
+		} else {
+			contents[contentsDes.repetitions] = '\0';
+			DoTransfer(contents);
+		}
+		FREE(contents);
 		break;
 
 #ifdef WITH_NETLOGGER	  
@@ -967,164 +1084,71 @@ static struct loglocation memLogLocation;
 #define DEFAULT_MEMORY_DIR "."
 #define DEFAULT_MEMORY_SIZE "2000"
 #define DEFAULT_JOURNAL_SIZE "2000"
-#define SWITCHES "C:a:d:n:e:l:N:Pp:s:v:Vi:B:"
 
 static void usage() {
-	printf("\nUsage: nws_memory [OPTIONS]\n");
-	printf("Memory for the Network Weather Service\n");
-	printf("\nOPTIONS can be:\n");
-	printf("\t-e filename         write error messages to filename\n");
-	printf("\t-l filename         write info/debug messages to filename\n");
-	printf("\t-i filename         write pid to filename\n");
-	printf("\t-a address          use this address as mine (ie multi-homed hosts)\n");
-	printf("\t-n name             use name as my hostname\n");
-	printf("\t-N nameserver       register with this nameserver\n");
-	printf("\t-p port             bind to port instead of the default\n");
+	DaemonUsage("nws_memory", "Memory");
 	printf("\t-s num              keep #num# measurement per experiment\n");
 	printf("\t-d dir              save series files into directory dir\n");
 	printf("\t-B db               use database #db# to store data\n");
+	printf("\t-I idle             remove series unmodified for #idle# seconds\n");
 	printf("\t-C #                # of caches to keep (0 to disable cache)\n");
-	printf("\t-v level            verbose level (up to 5)\n");
-	printf("\t-V                  print version\n");
-	printf("Report bugs to <nws@nws.cs.ucsb.edu>.\n\n");
 }
 
 int
 main(		int argc,
 		char *argv[]) {
-	IPAddress addresses[MAX_ADDRESSES], tmpAddress;
-	unsigned int addressesCount;
-	struct host_cookie memCookie,
-		nsCookie;
-	double nextBeatTime, wakeup;
+	double nextBeatTime, wakeup, nextClean;
 	double now;
 	int opt, 
-		tmp,
-		cacheEntries = 256,
-		verbose;
+		cacheEntries,
+		autoIdle,
+		ind;
 	extern char *optarg;
-	char tmpIP[127 + 1],
-		password[127 + 1],
-		memoryDir[255 + 1];
-	FILE *logFD, *errFD;
-	const char *c;
-	char *toProcess, *dbname, *pidFile;
+	char memoryDir[255 + 1];
+	char *toProcess, *dbname;
 
 	/* Set up default values */
-	verbose = 2;
-	logFD = stdout;
-	errFD = stderr;
-	addressesCount = 0;
-	toProcess = pidFile = NULL;
+	toProcess = NULL;
+	cacheEntries = 256;
+	autoIdle = 0;
 
-	dbname = GetEnvironmentValue("MEMORY_SIZE", "~", ".nwsrc", DEFAULT_MEMORY_SIZE);
+	dbname = NWSAPI_EnvironmentValue_r("MEMORY_SIZE", DEFAULT_MEMORY_SIZE);
 	if (dbname == NULL) {
-		fileSize = strtol(DEFAULT_MEMORY_SIZE, NULL, 10);
-	} else {
-		fileSize = strtol(dbname, NULL, 10);
-		FREE(dbname);
-	}
-	dbname = GetEnvironmentValue("MEMORY_DIR", "~", ".nwsrc", DEFAULT_MEMORY_DIR);
+		ABORT("out of memory\n");
+	} 
+	fileSize = strtol(dbname, NULL, 10);
+	FREE(dbname);
+
+	dbname = NWSAPI_EnvironmentValue_r("MEMORY_DIR", DEFAULT_MEMORY_DIR);
 	if (dbname == NULL) {
-		SAFESTRCPY(memoryDir, DEFAULT_MEMORY_DIR);
-	} else {
-		SAFESTRCPY(memoryDir, dbname);
-		FREE(dbname);
+		ABORT("out of memory\n");
 	}
-	dbname = GetEnvironmentValue("NAME_SERVER", "~", ".nwsrc", "localhost");
-	if (dbname == NULL) {
-		SAFESTRCPY(nsCookie.name, "localhost");
-	} else {
-		SAFESTRCPY(nsCookie.name, dbname);
-		FREE(dbname);
-	}
-	Host2Cookie(nsCookie.name, DefaultHostPort(NAME_SERVER_HOST), &nsCookie);
-	password[0] = '\0';
+	SAFESTRCPY(memoryDir, dbname);
+	FREE(dbname);
 	dbname = NULL;
 
-	/* get my deafult name */
-	Host2Cookie(MyMachineName(), DefaultHostPort(MEMORY_HOST), &memCookie);
-	addressesCount = IPAddressValues(MyMachineName(),
-			&addresses[0],
-			MAX_ADDRESSES);
-	if (addressesCount == 0) {
-		ERROR1("Couldn't resolve my name (%s)\n", MyMachineName());
-	}
+	/* no error from getopt */
+	opterr = 0;
+	ind = InitializeDaemonStructure(MEMORY_HOST, "I:C:d:s:B:", MyExit);
+
+	while((int)(opt = getopt(argc, argv, DaemonAllSwitches(ind))) != EOF) {
+		if (DaemonSwitch(opt, optarg, ind)) {
+			continue;
+		}
 
-	while((int)(opt = getopt(argc, argv, SWITCHES)) != EOF) {
 		switch(opt) {
-		case 'n':
-			/* let's check we have a good name */
-			if (!IPAddressValue(optarg, &tmpAddress)) {
-				/* save the inet addresses */
-				addressesCount += IPAddressValues(optarg, &addresses[addressesCount], MAX_ADDRESSES - addressesCount);
-			} else {
-				ERROR1("Unable to convert '%s': I'll do what you said but expect problems!\n", optarg);
-			}
-
-			/* overrride the name of the machine: we hope the
-			 * user knows what is doing! */
-			SAFESTRCPY(memCookie.name, optarg);
-
-			break;
-
-		case 'a':
-			/* let's add this IPs to the list of my addresses */
-			for (c = optarg; GETTOK(tmpIP, c, ",", &c); ) {
-				tmp = IPAddressValues(tmpIP,
-						&addresses[addressesCount],
-						MAX_ADDRESSES - addressesCount);
-				if (tmp == 0) {
-					ERROR1("Unable to convert '%s' into an IP address\n", tmpIP);
-				} else {
-					addressesCount += tmp;
-				}
-			}
-			break;
-
 		case 'd':
 			SAFESTRCPY(memoryDir, optarg);
 			break;
 
-		case 'e':
-			/* open the error file */
-			errFD = fopen(optarg, "w");
-			if (errFD == NULL) {
-				printf("Couldn't open %s!\n", optarg);
-				exit(1);
+		case 'I':
+			autoIdle = (unsigned int)strtol(optarg, NULL, 10);
+			if (autoIdle < 300) {
+				INFO("main: too short idle: setting to 300\n");
+				autoIdle = 300;
 			}
 			break;
 
-		case 'l':
-			/* open the error file */
-			logFD = fopen(optarg, "w");
-			if (logFD == NULL) {
-				printf("Couldn't open %s!\n", optarg);
-				exit(1);
-			}
-			break;
-
-		case 'i':
-			/* write pid to file */
-			pidFile = strdup(optarg);
-			if (pidFile == NULL) {
-				ABORT("out of memory\n");
-			}
-			break;
-
-		case 'N':
-			Host2Cookie(optarg, DefaultHostPort(NAME_SERVER_HOST), &nsCookie);
-			break;
-
-		case 'p':
-			memCookie.port = strtol(optarg, NULL, 10);
-			break;
-
-		case 'P':
-			fprintf(stdout, "Password? ");
-			fscanf(stdin, "%s", password);
-			break;
-
 		case 's':
 			fileSize = strtol(optarg, NULL, 10);
 			break;
@@ -1140,34 +1164,12 @@ main(		int argc,
 #else
 			fprintf(stderr, "database is not enabled: ignoring -B\n");
 #endif
+			break;
 
 		case 'C':
 			cacheEntries = strtol(optarg, NULL, 10);
 			break;
 
-		case 'V':
-			printf("nws_memory for NWS version %s", VERSION);
-#ifdef HAVE_PTHREAD_H
-			printf(", with thread support");
-#endif
-#ifdef WITH_DEBUG
-			printf(", with debug support");
-#endif
-			printf("\n\n");
-			exit(0);
-			break;
-
-		case 'v':
-			verbose = (unsigned short)atol(optarg);
-			break;
-
-		case '?':
-			if (optopt == 'v') {
-				/* using the first level */
-				verbose = 1;
-				break;
-			}
-
 		default:
 			usage();
 			exit(1);
@@ -1180,9 +1182,6 @@ main(		int argc,
 	fetchTime = storeTime = oldStoreTime = autoFetchTime = 0;
 	fetchQuery = storeQuery = oldStoreQuery = 0;
 
-	/* let's set the verbose evel */
-	SetDiagnosticLevel(verbose, errFD, logFD);
-
 	/* now initializing the backend: if we use the directory, we need
 	 * to initialize the caching schema */
 	/* let's see if we are trying to use a database */
@@ -1214,41 +1213,16 @@ main(		int argc,
 	}
 
 	/* initialize the mySeries/orphanedSeries structures */
-	if (!InitRegistrations(&orphanedSeries)) {
-		ABORT("main: out of memory\n");
-	}
-	if (!InitRegistrations(&mySeries)) {
+	if (!InitRegistrations(&orphanedSeries) 
+			|| !InitRegistrations(&mySeries)) {
 		ABORT("main: out of memory\n");
 	}
 
 	/* let's get the port and start serving */
-	if(!EstablishHost(HostCImage(&memCookie),
-                    MEMORY_HOST,
-                    addresses,
-                    addressesCount,
-                    memCookie.port,
-                    password,
-                    &nsCookie,
-		    NULL,		/* we are the memory ... */
-                    &MyExit)) {
-		ABORT("Unable to establish host: port already in use?\n");
+	if (!InitializeDaemon(0, 1, 0, ind)) {
+		exit(-1);
 	}
 
-	/* now that we've got the port, we can print the pid into the
-	 * pid file. We thus avoid to override pid files of running
-	 * nameservers */
-	if (pidFile != NULL) {
-		FILE *pidfile = fopen(pidFile, "w");
-		if (!pidfile) {
-			ABORT1("Can't write pidfile %s\n", pidFile);
-		}
-		fprintf(pidfile, "%d", (int)getpid());
-		fclose(pidfile);
-		free(pidFile);
-	}
-
-	fclose(stdin);
-
 	RegisterListener(STORE_STATE, "STORE_STATE", &ProcessRequest);
 	RegisterListener(STORE_AND_REGISTER, "STORE_AND_REGISTER", &ProcessRequest);
 	RegisterListener(FETCH_STATE, "FETCH_STATE", &ProcessRequest);
@@ -1261,7 +1235,7 @@ main(		int argc,
 		WARN("main: failed to register for disconnections\n");
 	}
 
-	nextBeatTime = 0;
+	nextClean = nextBeatTime = 0;
 
 	/* main service loop */
 	while(1) {
@@ -1312,6 +1286,13 @@ main(		int argc,
 			wakeup = -1;
 		}
 
+		/* let's see if we need to clean older series */
+		if (autoIdle > 0 && nextClean < now) {
+			nextClean = now + autoIdle;
+			DoClean("", autoIdle);
+			LOG("main: autocleaning\n");
+		}
+
 		/* we want the memory to be repsonsive, so we flush all
 		 * the waiting messages */
 		while(ListenForMessages(-1)) {
