@@ -1,4 +1,4 @@
-/* $Id: nws_nameserver.c,v 1.182 2004/11/04 21:25:18 graziano Exp $ */
+/* $Id: nws_nameserver.c,v 1.193 2005/08/04 21:51:23 graziano Exp $ */
 
 #include "config_nws.h"
 
@@ -18,6 +18,7 @@
 #include "register.h"
 #include "nws_nameserver.h"
 #include "timeouts.h"
+#include "nws_daemon.h"
 
 /*
  * This program implements the NWS name server host.  See nws_messages.h for
@@ -78,7 +79,7 @@ AddSetToRegistrations(	ObjectSet objs,
 		}
 
 		/* let's set the timeout */
-		timeOut = (expTime  > DEFAULT_HOST_BEAT*2) ? expTime : DEFAULT_HOST_BEAT*2;
+		timeOut = now + ((expTime  > DEFAULT_HOST_BEAT*2) ? expTime : DEFAULT_HOST_BEAT*2);
 
 
 		/* we don't need to keep this one ordered: we keep adding
@@ -114,7 +115,7 @@ ReadRegistrationFile(	const char *name,
 
 	/* starting registration size: we'll increase it if necessary */
 	size = 1024;
-	registration = (char *)MALLOC(sizeof(char)*size);
+	registration = (char *)MALLOC(sizeof(char)*size, 0);
 	if (registration == NULL) {
 		ERROR("ReadRegistrationFile: out of memory\n");
 		fclose(regFile);
@@ -130,7 +131,7 @@ ReadRegistrationFile(	const char *name,
 		if (len >= size-1) {
 			/* enlarge buffer */
 			size += 1024;
-			registration = (char *)REALLOC((void *)registration, sizeof(char)*size);
+			registration = (char *)REALLOC((void *)registration, sizeof(char)*size, 0);
 			if (registration == NULL) {
 				ERROR("ReadRegistrationFile: out of memory\n");
 				fclose(regFile);
@@ -212,6 +213,7 @@ Dump2File(	registrations *r,
 static int
 EmergencyExit() {
 	Dump2File(reg, registrationFile);
+
 	return 1;
 }
 
@@ -498,11 +500,6 @@ DoSearchFrom(	const char *filter,
 
 	return ret;
 }
-static int 
-DoSearch(	const char *filter, 
-		int expired) {
-	return DoSearchFrom(filter, 0, expired);
-}
 
 /*
  * this are search shortcuts. We recognizes only few patterns. Returns
@@ -542,10 +539,7 @@ SearchShortcuts(	const char *filter,
 			len1 = strlen(reg->vals[i]);
 
 			/* make room */
-			*whereTo = (char *)REALLOC(*whereTo, len + len1 +1);
-			if (*whereTo == NULL) {
-				ABORT("SearchShortcuts: out of memory");
-			}
+			*whereTo = (char *)REALLOC(*whereTo, len + len1+1, 1);
 
 			/* copy the element */
 			memcpy(*whereTo + len, reg->vals[i], len1 + 1);
@@ -566,10 +560,7 @@ SearchShortcuts(	const char *filter,
 		/* and add a timestamp */
 		sprintf(tmp, "%10ld", now + DEFAULT_HOST_BEAT*2);
 		AddNwsAttribute(&obj, "timestamp", tmp);
-		if (!AddObject(whereTo, obj)) {
-			/* we shouldn't get here */
-			ABORT("SearchShortcuts: out of memory?");
-		}
+		AddObject(whereTo, obj);
 		FREE(obj);
 
 		for (i = 0; i < NScache->howMany; i++) {
@@ -579,10 +570,7 @@ SearchShortcuts(	const char *filter,
 			}
 
 			/* add the object */
-			if (!AddObject(whereTo, NScache->reg[i])) {
-				/* we shouldn't get here */
-				ABORT("SearchShortcuts: out of memory?");
-			}
+			AddObject(whereTo, NScache->reg[i]);
 		}
 		len = strlen(*whereTo);		/* returns the # of char */
 	} else if (strncmp(filter, "(&(name=", 8) == 0) {
@@ -610,14 +598,9 @@ SearchShortcuts(	const char *filter,
 					/* let's check we still have a
 					 * similar name */
 					ptr = NwsAttributeValue_r(FindNwsAttribute(reg->vals[i], "name"));
-					if (ptr == NULL) {
-						ABORT("SearchShortcuts: out of memory?");
-					}
 					if (strcmp(ptr, name) == 0) {
 						/* add the object */
-						if (!AddObject(whereTo, reg->vals[i])) {
-							ABORT("SearchShortcuts: out of memory?");
-						}
+						AddObject(whereTo, reg->vals[i]);
 					}
 					FREE(ptr);
 				}
@@ -649,14 +632,9 @@ SearchShortcuts(	const char *filter,
 					/* let's check we still have a
 					 * similar name */
 					ptr = NwsAttributeValue_r(FindNwsAttribute(reg->vals[i], "name"));
-					if (ptr == NULL) {
-						ABORT("SearchShortcuts: out of memory?");
-					}
 					if (strcmp(ptr, name) == 0) {
 						/* add the object */
-						if (!AddObject(whereTo, reg->vals[i])) {
-							ABORT("SearchShortcuts: out of memory?");
-						}
+						AddObject(whereTo, reg->vals[i]);
 					}
 					FREE(ptr);
 				}
@@ -774,17 +752,15 @@ CheckNScache(	Object registration,
 	if (i >= NScache->howMany) {
 		/* let's add the entry */
 		NScache->howMany++;
-		NScache->reg = (char **)REALLOC(NScache->reg, NScache->howMany*(sizeof(char *)));
-		NScache->cookie = (struct host_cookie *)REALLOC(NScache->cookie, NScache->howMany*(sizeof(struct host_cookie)));
-		NScache->last = (unsigned long *)REALLOC(NScache->last, NScache->howMany*(sizeof(unsigned long)));
-		NScache->queue = (registrations *)REALLOC(NScache->queue, NScache->howMany*(sizeof(registrations)));
-		NScache->deQueue = (registrations *)REALLOC(NScache->deQueue, NScache->howMany*(sizeof(registrations)));
-		if (NScache->reg == NULL || NScache->cookie == NULL || NScache->queue == NULL || NScache->deQueue == NULL) {
-			ABORT("CheckNScache: out of memory");
-		}
+		NScache->reg = (char **)REALLOC(NScache->reg, NScache->howMany*(sizeof(char *)), 1);
+		NScache->cookie = (struct host_cookie *)REALLOC(NScache->cookie, NScache->howMany*(sizeof(struct host_cookie)), 1);
+		NScache->last = (unsigned long *)REALLOC(NScache->last, NScache->howMany*(sizeof(unsigned long)), 1);
+		NScache->queue = (registrations *)REALLOC(NScache->queue, NScache->howMany*(sizeof(registrations)), 1);
+		NScache->deQueue = (registrations *)REALLOC(NScache->deQueue, NScache->howMany*(sizeof(registrations)), 1);
 	}
 
 	/* get the name in the cookie */
+	NScache->reg[i] = NULL;
 	NScache->cookie[i].name[0] = '\0';
 	NScache->cookie[i].port = 0;
 	NScache->cookie[i].sd = sd;
@@ -943,10 +919,12 @@ DoRegister(	char *object,
  */
 static int
 DoUnregister(const char *filter) {
-	int j, i;
-	char *name, *ptr;
+	int i, ret;
+	char *name, *ptr, *tmp;
 
+	ret = 0;
 	i = -1;
+
 	/* let's try to optimize: most of the unregistrations comes with
 	 * a filter like (name=%s): this is great since we have the
 	 * registrations ordered that way. If so use binary search */
@@ -956,32 +934,44 @@ DoUnregister(const char *filter) {
 			ABORT("DoUnregister: out of memory\n");
 		}
 		ptr = strchr(name, ')');
-		if (ptr && strcmp(ptr, ")") == 0) {
-			/* got it */
+		if (ptr) {
 			*ptr = '\0';
-			if (!SearchForName(reg, name, 1, &i)) {
-				i = -1;
+
+			if (SearchForName(reg, name, 1, &i)) {
+				int done;
+
+				/* let's find all the entries */
+				do {
+					tmp = NwsAttributeValue_r(FindNwsAttribute(reg->vals[i], "name"));
+					done = strcmp(tmp, name);
+					if (done == 0) {
+						ret = 1;
+						reg->expirations[i] = EXPIRED;
+					}
+					FREE(tmp);
+					i--;
+				} while (done == 0);
 			}
+			/* no normal search after this */
+			i = reg->howMany;
 		}
 		FREE(name);
 	}
-				
-	if (i == -1) {
+	do {
 		/* do the normal search */
-		j = DoSearch(filter, 0);
-	} else {
-		j = i;
-	}
+		i = DoSearchFrom(filter, i + 1, 0);
+		if (i != -1) {
+			/* set EXPIRED to the expiration field */
+			ret = 1;
+			reg->expirations[i] = EXPIRED;
+		}
+	} while (i != -1);
 
-	if (j == -1) {
+	if (ret == 0) {
 		INFO("DoUnregister: couldn't find a matching registration\n");
-		return 0;
-	} else {
-		/* set EXPIRED to the expiration field */
-		reg->expirations[j] = EXPIRED;
 	}
 
-	return 1;
+	return ret;
 }
 
 /*
@@ -1109,10 +1099,7 @@ AddToNSqueue(	Object registration,
 			/* since this is a pattern we add a
 			 * fromNameserver in front of the pattern to
 			 * specify it's from a nameserver */
-			NScache->deQueue[i].vals[j] = (char *)MALLOC(strlen(obj) + FROM_NS_LEN + 1);
-			if (NScache->deQueue[i].vals[j] == NULL) {
-				ABORT("AddToNSqueue: out of memory\n");
-			}
+			NScache->deQueue[i].vals[j] = (char *)MALLOC(strlen(obj) + FROM_NS_LEN + 1, 1);
 			NScache->deQueue[i].vals[j][0] = '\0';
 			strcat(NScache->deQueue[i].vals[j], FROM_NS);
 			strcat(NScache->deQueue[i].vals[j], obj);
@@ -1244,7 +1231,7 @@ ProcessRequest(	Socket *sd,
 		registerTime -= MicroTime();		/* start the timer */
 
 		objectDescriptor.repetitions = header.dataSize - HomogenousDataSize(UNSIGNED_LONG_TYPE, 1, NETWORK_FORMAT);
-		object = (char *)MALLOC(objectDescriptor.repetitions + 1);
+		object = (char *)MALLOC(objectDescriptor.repetitions + 1, 0);
 
 		/* sanity check */
 		if(object == NULL) {
@@ -1318,7 +1305,7 @@ ProcessRequest(	Socket *sd,
 
 		/* let's receive the string and be sure to NULL terminate it */
 		patternDescriptor.repetitions = header.dataSize;
-		pattern = (char *)MALLOC(header.dataSize + 1);
+		pattern = (char *)MALLOC(header.dataSize + 1, 0);
 		if (pattern == NULL) {
 			SendMessage(*sd, NS_FAILED, -1);
 			DROP_SOCKET(sd);
@@ -1364,7 +1351,7 @@ ProcessRequest(	Socket *sd,
 		searchQuery++;			/* add the # of query */
 
 		patternDescriptor.repetitions = header.dataSize;
-		pattern = (char *)MALLOC(header.dataSize + 1);
+		pattern = (char *)MALLOC(header.dataSize + 1, 0);
 		if(pattern == NULL) {
 			(void)SendMessage(*sd, NS_FAILED, -1);
 			DROP_SOCKET(sd);
@@ -1401,7 +1388,7 @@ ProcessRequest(	Socket *sd,
 			int temp;
 
 			temp = strlen(reg->vals[matching]);
-			object = (char *)REALLOC(object, last + temp + 1);
+			object = (char *)REALLOC(object, last + temp + 1, 0);
 			if (object == NULL) {
 				break;
 			} 
@@ -1479,7 +1466,7 @@ ProcessRequest(	Socket *sd,
 	case NS_SEARCHED:
 		/* this comes from a nameserver we want to sync with */
 		objectDescriptor.repetitions = header.dataSize;
-		object = (char *)MALLOC(header.dataSize + 1);
+		object = (char *)MALLOC(header.dataSize + 1, 0);
 		if (object == NULL) {
 			DROP_SOCKET(sd);
 			ERROR("ProcessRequest: out of memory\n");
@@ -1498,7 +1485,7 @@ ProcessRequest(	Socket *sd,
 		break;
 
 	case HOST_REGISTER:
-		object = (char *)MALLOC(header.dataSize + 1);
+		object = (char *)MALLOC(header.dataSize + 1, 0);
 		if (object == NULL) {
 			DROP_SOCKET(sd);
 			ERROR("ProcessRequest: out of memory\n");
@@ -1607,101 +1594,44 @@ SyncUP( 	registrations *r,
 	return (r->howMany > 0);
 }
 
-#define SWITCHES "a:c:mDe:f:l:p:PN:v:Vi:n:"
 
 static void 
 usage() {
-	printf("\nUsage: nws_nameserver [OPTIONS]\n");
-	printf("Nameserver for the Network Weather Service\n");
-	printf("\nOPTIONS can be:\n");
-	printf("\t-e filename         write error messages to filename\n");
-	printf("\t-l filename         write info/debug messages to filename\n");
-	printf("\t-i filename         write pid to filename\n");
+	DaemonUsage("nws_nameserver", "Nameserver");
 	printf("\t-f filename         use filename as registration s file\n");
-	printf("\t-p port             bind to port instead of the default\n");
-	printf("\t-a address          use this address as mine (ie multi-homed hosts)\n");
-	printf("\t-n name             use name as my hostname\n");
-	printf("\t-m                  enable EXPERIMENTAL replica mode\n");
+	printf("\t-m                  enable replica mode\n");
 	printf("\t-c time             clean registrations file every time seconds\n");
-	printf("\t-v level            verbose level (up to 5)\n");
-	printf("\t-V                  print version\n");
-	printf("\t-N nameserver       experimental!\n");
-	printf("\t-D                  obsolete (turns debugging on)\n\n");
-	printf("Report bugs to <nws@nws.cs.ucsb.edu>.\n\n");
 }
 
 int
 main(	int argc,
 	char *argv[]) {
 
-	IPAddress addresses[MAX_ADDRESSES], tmpAddress;
-	unsigned int addressesCount;
 	unsigned long compressionFreq,	/* how often to clean registrations */
 		nextBeatTime, 
 		nextCompress, 
 		nextWait,
 		now; 
 	double wakeup;
-	struct host_cookie nsCookie = {"", 0, NO_SOCKET},
-		myCookie;
-	char password[127 + 1], tmpIP[127+1], *pidFile;
-	int opt, verbose = 2, tmp;
+	int opt, tmp, ind;
 	extern char *optarg;
-	const char *c;
-	FILE *logFD, *errFD;
 
 	/* Set up default values to be overwritten by command line args. */
 	compressionFreq = DEFAULT_COMPRESSION_FREQUENCY;
 	SAFESTRCPY(registrationFile, DEFAULT_FILE);
-	password[0] = '\0';
-	logFD = stdout;
-	errFD = stderr;
-	pidFile = NULL;
 	doForward = 0;			/* default: no mirroring */
 
-	/* get my deafult name first */
-	Host2Cookie(MyMachineName(), DefaultHostPort(NAME_SERVER_HOST), &myCookie);
-	addressesCount = IPAddressValues(MyMachineName(),
-			&addresses[0],
-			MAX_ADDRESSES);
-	if (addressesCount == 0) {
-		ERROR1("Couldn't resolve my name (%s)\n", MyMachineName());
-	} 
-
 	/* no error messages from getopt() */
 	opterr = 0;
+	ind = InitializeDaemonStructure(NAME_SERVER_HOST, "f:mc:", EmergencyExit);
+
+	while((int)(opt = getopt(argc, argv, DaemonAllSwitches(ind))) != EOF) {
+		if (DaemonSwitch(opt, optarg, ind)) {
+			/* we already dealt with it */
+			continue;
+		}
 
-	while((int)(opt = getopt(argc, argv, SWITCHES)) != EOF) {
 		switch(opt) {
-		case 'n':
-			/* let's check we have a good name */
-			if (IPAddressValue(optarg, &tmpAddress)) {
-				/* save the inet addresses */
-				addressesCount += IPAddressValues(optarg, &addresses[addressesCount], MAX_ADDRESSES - addressesCount);
-			} else {
-				ERROR1("Unable to resolve '%s': I'll do what you said but expect problems!\n", optarg);
-			}
-
-			/* overrride the name of the machine: we hope the
-			 * user knows what is doing! */
-			SAFESTRCPY(myCookie.name, optarg);
-
-			break;
-
-		case 'a':
-			/* let's add this IPs to the list of my addresses */  
-			for (c = optarg; GETTOK(tmpIP, c, ",", &c); ) {
-				tmp = IPAddressValues(tmpIP,
-						&addresses[addressesCount],
-						MAX_ADDRESSES - addressesCount);
-				if (tmp == 0) {
-					ERROR1("Unable to convert '%s' into an IP address\n", tmpIP);
-				} else {
-					addressesCount += tmp;
-				}
-			}
-			break;
-
 		case 'c':
 			tmp = atol(optarg);
 			if (tmp != 0) {
@@ -1709,80 +1639,14 @@ main(	int argc,
 			}
 			break;
 
-		case 'D':
-			verbose = 5;
-			break;
-
-		case 'e':
-			/* open the error file */
-			errFD = fopen(optarg, "w");
-			if (errFD == NULL) {
-				printf("Couldn't open %s!\n", optarg);
-				exit(1);
-			}
-			break;
-
 		case 'f':
 			SAFESTRCPY(registrationFile, optarg);
 			break;
 
-		case 'l':
-			/* open the error file */
-			logFD = fopen(optarg, "w");
-			if (logFD == NULL) {
-				printf("Couldn't open %s!\n", optarg);
-				exit(1);
-			}
-			break;
-
-		case 'i':
-			pidFile = strdup(optarg);
-			if (pidFile == NULL) {
-				ABORT("out of memory1\n");
-			}
-			break;
-
 		case 'm':
 			doForward = 1;		/* mirroring enabled */
 			break;
 
-		case 'N':
-			SAFESTRCPY(nsCookie.name, optarg);
-			break;
-
-		case 'p':
-			myCookie.port = strtol(optarg, NULL, 10);
-			break;
-
-		case 'P':
-			fprintf(stdout, "Password? ");
-			fscanf(stdin, "%s", password);
-			break;
-
-		case 'v':
-			verbose = (unsigned short)atol(optarg);
-			break;
-
-		case 'V':
-			printf("nws_nameserverv for NWS version %s", VERSION);
-#ifdef HAVE_PTHREAD_H
-			printf(", with thread support");
-#endif
-#ifdef WITH_DEBUG
-			printf(", with debug support");
-#endif
-			printf("\n\n");
-			exit(0);
-			break;
-
-		case '?':
-			if (optopt == 'v') {
-				/* using the first level */
-				verbose = 2;
-				break;
-			}
-			/* non-recognized options: printing help */
-
 		default:
 			usage();
 			exit(1);
@@ -1796,10 +1660,7 @@ main(	int argc,
 	}
 	
 	/* initialize the NameServers cache */
-	NScache = (nameServerCache *) MALLOC(sizeof(nameServerCache));
-	if (NScache == NULL) {
-		ABORT("main: out of memory");
-	}
+	NScache = (nameServerCache *) MALLOC(sizeof(nameServerCache), 1);
 	NScache->howMany = 0;
 	NScache->reg = NULL;
 	NScache->last = NULL;
@@ -1822,10 +1683,6 @@ main(	int argc,
 		childs[tmp].pid = 0;
 	}
 
-	/* let's set the verbose level  and open up files (if needed) */
-	/* fatal errors are always on */
-	SetDiagnosticLevel(verbose, errFD, logFD);
-
 	/* setting the defaults timeouts: we use tight timeouts to not
 	 * block on slow machines  */
 	SetDefaultTimeout(CONN, MIN_TIME_OUT, MAX_TIME_OUT/3, TIMEOUT_TIME_OUT);
@@ -1833,51 +1690,26 @@ main(	int argc,
 	SetDefaultTimeout(SEND, MIN_TIME_OUT, MAX_TIME_OUT, TIMEOUT_TIME_OUT);
 
 	/* let's retrieve the list of nameservers */
-	if (nsCookie.name[0] != '\0') {
+	if (doForward) {
 		ObjectSet objs = NULL;
 
-		Host2Cookie(nsCookie.name, DefaultHostPort(NAME_SERVER_HOST), &nsCookie);
-		if (RetrieveObjects(&nsCookie, "hostType=nameserver", &objs, -1)) {
+		if (RetrieveFromMyNameserver("hostType=nameserver", &objs)) {
 			AddSetToRegistrations(objs, 1, toSync);
-			FREE(objs);
 		} else {
-			ERROR1("main: failed to talk to %s\n", nsCookie.name);
+			ERROR("main: failed to sync with nameserver\n");
 		}
-		DisconnectHost(&nsCookie);
+		FREE(objs);
 	}
 
 	/* let's read the old registrations if the file is there */
 	ReadRegistrationFile(registrationFile, toSync);
 
 	/* ready to do work now */
-	if(!EstablishHost(HostCImage(&myCookie),
-			NAME_SERVER_HOST,
-			addresses,
-			addressesCount,
-			myCookie.port,
-			password,
-			&myCookie,
-			NULL,			/* no memory for us */
-			EmergencyExit)) {
-		ERROR("Unable to establish host: port already in use?\n");
+	if (!InitializeDaemon(0, 0, 0, ind)) {
 		exit(-1);
 	}
 	
-	/* now that we've got the port, we can print the pid into the
-	 * pid file. We thus avoid to override pid files of running
-	 * nameservers */
-	if (pidFile != NULL) {
-		FILE *pidfile = fopen(pidFile, "w");
-		if (!pidfile) {
-			ABORT1("Can't write pidfile %s\n", pidFile);
-		}
-		fprintf(pidfile, "%d", (int)getpid());
-		fclose(pidfile);
-		free(pidFile);
-	}
-
 	/* let's get ready to work */
-	fclose(stdin);
 	RegisterListener(NS_REGISTER, "NS_REGISTER", &ProcessRequest);
 	RegisterListener(NS_SEARCH, "NS_SEARCH", &ProcessRequest);
 	RegisterListener(NS_UNREGISTER, "NS_UNREGISTER", &ProcessRequest);
@@ -1886,7 +1718,7 @@ main(	int argc,
 
 	/* we deal ourself with the HOST_REGISTER */
 	UnregisterListener(HOST_REGISTER);
-	RegisterListener(HOST_REGISTER, "NS_SEARCHED", &ProcessRequest);
+	RegisterListener(HOST_REGISTER, "HOST_REGISTER", &ProcessRequest);
 
 	nextWait = nextBeatTime = nextCompress = 0;
 
