@@ -1,4 +1,4 @@
-/* $Id: nws_proxy.c,v 1.50 2004/12/02 04:10:54 graziano Exp $ */
+/* $Id: nws_proxy.c,v 1.92 2005/07/08 17:41:17 graziano Exp $ */
 
 #include "config_nws.h"
 
@@ -7,13 +7,17 @@
 
 #include "diagnostic.h"
 #include "protocol.h"
+#include "host_protocol.h"
 #include "osutil.h"
 #include "dnsutil.h"
 #include "messages.h"
+#include "register.h"
 #include "skills.h"
 #include "strutil.h"
 #include "nws_proxy.h"
+#include "protocol.h"
 #include "nws_api.h"
+#include "nws_daemon.h"
 
 /* we have here the list of series (and respective memories) that we are
  * keeping track of. We don't use the timeout (in registrations) so we
@@ -31,6 +35,7 @@ static char *getOld, *lost;
 static int getOldLen, lostLen;
 static NWSAPI_ForecastState **forecasts;
 static int highestForecastIndex = 0;
+static int local = 1;
 
 /* performance counter */
 static long updates = 0;
@@ -66,6 +71,24 @@ DeleteSingleForecast(	const char *seriesName) {
 }
 #endif
 
+/* trap SIGTERM */
+static int 
+MyExit(void) {
+	return 1;
+}
+
+/* retireve data from a nameserver. If we are not local we can call our
+ * nameserver (the one we register with) but if we are local, we need to
+ * call the nws api. */
+static int
+GetFromNS(	const char *filter,
+		ObjectSet *whereTo) {
+	if (!local) {
+		return RetrieveFromMyNameserver(filter, whereTo);
+	}
+
+	return NWSAPI_GetObjects(filter, whereTo);
+}
 
 /* add a forecaster */
 static int
@@ -106,7 +129,7 @@ AddSingleForecast(	const Object seriesName) {
 	}
 	if (j >= highestForecastIndex) {
 		/* make room for the forecaster */
-		forecasts = REALLOC(forecasts, sizeof(NWSAPI_ForecastState **)*(highestForecastIndex + 1));
+		forecasts = REALLOC(forecasts, sizeof(NWSAPI_ForecastState **)*(highestForecastIndex + 1), 0);
 		if (forecasts == NULL) {
 			ReleaseNWSLock(&lock);
 			ERROR("AddSingleForecaster: out of memory\n");
@@ -189,26 +212,125 @@ RestartForecast(	const char *name) {
 	return 1;
 }
 
+int
+FindMeasurement(	char *host,
+			char *dst,
+			char *resource,
+			char *opts,
+			NWSAPI_ForecastCollection *col) {
+	int i, done, ret;
+	char *name, *ptr, src[MAX_HOST_NAME + 8];
+
+	/* we might change the source, so we copy it */
+	if (strlen(host) > MAX_HOST_NAME + 4) {
+		ERROR1("FindMeasurement: host name too long (%s)\n", host);
+		return 0;
+	}
+	strcpy(src, host);
+
+	/* let's try a simple search now: we do a non-exact match beacuse
+	 * we could specify the host without the port. */
+	done = 0;
+	if (!SearchForName(series, src, 0, &i)) {
+		/* we didn't find it: let's try to be more aggressive
+		 * with the name in case there is a port but not a domain
+		 * name*/
+		ptr = strchr(src, ':');
+		if (!(ptr && strchr(src, '.') == NULL)) {
+			return 0;
+		}
+
+		INFO1("FindMeasurement: %s has port and not domain name: expanding the search\n", src);
+		name = MALLOC(strlen(src) + 3, 1);
+
+		/* now add .* before the : */
+		ret = ptr - src;
+		memcpy(name, src, ret);
+		name[ret++] = '.';
+		name[ret++] = '*';
+		memcpy(name + ret, ptr, strlen(ptr) + 1);
+		strcpy(src, name);
+		FREE(name);
+
+		/* now let's try it again with a sloppy match */
+		if (!SearchForName(series, src, 0, &i)) {
+			/* there is no such host */
+			return 0;
+		}
+	}
+
+	for (name = NULL; i >= 0; i--) {
+		/* we need to be sure that the registration is the right
+		 * one, so we have to look for the host, the target and
+		 * the resource. */
+
+		/* now let's check that the host is the right one .. */
+		name = NwsAttributeValue_r(FindNwsAttribute(series->vals[i], "host"));
+		if (name == NULL) {
+			continue;
+		}
+		ret = NWSAPI_HostCompare(name, host);
+		FREE(name);
+		if (!ret) {
+			/* wrong host */
+			break;
+		}
+
+		/* now let's check if the resource is right */
+		name = NwsAttributeValue_r(FindNwsAttribute(series->vals[i], "resource"));
+		if (name == NULL) {
+			continue;
+		}
+		ret = strcmp(name, resource);
+		FREE(name);
+		if (ret != 0) {
+			/* wrong resource */
+			continue;
+		}
+
+		/* now let's check the target */
+		if (NWSAPI_IntermachineResource(resource)) {
+			name = NwsAttributeValue_r(FindNwsAttribute(series->vals[i], "target"));
+			if (name == NULL) {
+				continue;
+			}
+			ret =  NWSAPI_HostCompare(name, dst);
+			FREE(name);
+			if (!ret) {
+				/* nope, wrong target */
+				continue;
+			}
+		}
+
+		/* got it! */
+		if (!NWSAPI_ComputeForecast(forecasts[series->expirations[i]], col)) {
+			ERROR("FindMeasurement: failed to compute forecasts!\n");
+		} else {
+			/* done for this round */
+			done = 1;
+			break;
+		}
+	}
+
+	return done;
+}
+
 /* used by ProcessRequest to service GET_FORECASTS. We returns the
  * forecasts in #col# for #resource# (possibly specifying which options
- * we'd like to have) for the #howMany#-long list of #hosts#. We returned
- * in #filled# how many forecasters we filled. */
+ * we'd like to have) for the #howMany#-long list of #hosts#. */
 static int
 GetAllForecasts(	char *resource,
 			char *opts,
 			char **hosts,
 			int howMany,
-			NWSAPI_ForecastCollection *col,
-			int *filled) {
-	int i, interMachine, j, z, len1, len2, ret, ind;
-	char *name;
+			NWSAPI_ForecastCollection *col) {
+	int i, interMachine, j;
 	
 	/* sanity check */
-	if (resource==NULL || hosts == NULL || col == NULL || filled == NULL) {
+	if (resource==NULL || hosts == NULL || col == NULL) {
 		ERROR("GetAllForecasts: NULL parameter(s)\n");
 		return 0;
 	}
-	*filled = 0;
 	if (NWSAPI_IsResourceValid(resource) == -1) {
 		WARN1("GetAllForecasts: unknown resource (%s)\n", resource);
 		return 0;
@@ -225,78 +347,30 @@ GetAllForecasts(	char *resource,
 
 	GetNWSLock(&lock);
 	/* let's go through all the hosts */
-	for (i = 0, ind = 0; i < howMany; i++) {
-		for (j = 0; j < howMany; j++) {
-			/* if we are asking for a single resource series,
-			 * we don't need to go further then j = 0 */
-			if (!interMachine && j > 0) {
-				break;
-			}
+	for (j = 0; j < howMany; j++) {
+		/* if we are asking for a single resource series,
+		 * we don't need to go further then j = 0 */
+		if (!interMachine && j > 0) {
+			break;
+		}
+		for (i = 0; i < howMany; i++) {
+			char *tmp;
 
 			/* set to not-good by default */
-			col[ind].measurement.timeStamp = 0;
-			(*filled)++;
-			ind++;
+			col[i + howMany*j].measurement.timeStamp = 0;
 
-			/* the series name will start with hosts[i] */
-			if (!SearchForName(series, hosts[i], 0, &z)) {
-				/* there is no such host */
-				continue;
-			}
-			len1 = strlen(hosts[i]);
-			len2 = strlen(hosts[j]);
-
-			for (name = NULL; z >= 0; z--) {
-				/* are we still talking about a possible
-				 * series (initial name matches)? */
-				name = NwsAttributeValue_r(FindNwsAttribute(series->vals[z], "name"));
-				if (name == NULL) {
-					ABORT("GetAllForecasts: out of memory\n");
-				}
-
-				/* let's check it still matches */
-				ret = strncmp(name, hosts[i], len1);
-				FREE(name);
-				if (ret != 0) {
-					/* wrong host */
-					break;
-				}
-
-				/* let's see if the resource is right */
-				name = NwsAttributeValue_r(FindNwsAttribute(series->vals[z], "resource"));
-				if (name == NULL) {
-					ABORT("GetAllForecasts: out of memory\n");
-				}
-				ret = strcmp(name, resource);
-				FREE(name);
-				if (ret != 0) {
-					/* wrong resource */
+			/* if we got interMachine and j == i we won't
+			 * find that measurement */
+			if (interMachine) {
+				if (i == j) {
 					continue;
 				}
-
-				/* now let's see if we need to check the
-				 * target */
-				if (interMachine) {
-					name = NwsAttributeValue_r(FindNwsAttribute(series->vals[z], "target"));
-					if (name == NULL) {
-						ABORT("GetAllForecasts: out of memory\n");
-					}
-					ret = strncmp(name, hosts[j], len2);
-					FREE(name);
-					if (ret != 0) {
-						/* nope, wrong target */
-						continue;
-					}
-				}
-
-				/* got it! */
-				if (!NWSAPI_ComputeForecast(forecasts[series->expirations[z]], &col[ind -1])) {
-					ERROR("GetAllForecasts: failed to compute forecasts!\n");
-				} else {
-					/* done for this round */
-					break;
-				}
+				tmp = hosts[j];
+			} else {
+				tmp = hosts[i];
 			}
+
+			FindMeasurement(tmp, hosts[i], resource, opts, &(col[i + howMany*j]));
 		}
 	}
 	ReleaseNWSLock(&lock);
@@ -304,13 +378,54 @@ GetAllForecasts(	char *resource,
 	return 1;
 }
 
-#define MY_FILTER "&(objectclass=nwsSeries)(resource="
+/* we go through the list of forecasts to find our which hosts we don't
+ * have. We could keep a global list, but we should do so with resources
+ * and options, so we keep it simple and build a list on the fly */
+static int
+CheckEmpty(	int interMachine,
+		char **hosts,
+		int howMany,
+		NWSAPI_ForecastCollection *col,
+		int **list) {
+	int i, j, soFar;
+
+	*list = (int *)MALLOC(sizeof(int) * howMany, 1);
+	(*list)[0] = -1;
+	soFar = 0;
+
+	for (i = 0; i < howMany; i++) {
+		for (j = 0; j < howMany; j++) {
+			/* only the interMachine experiments have a j > 0 */
+			if (!interMachine && j > 0) {
+				j = howMany;
+				break;
+			}
+
+			/* i = src and j = dst: do we have a measurement? */
+			if (col[i + howMany*j].measurement.timeStamp != 0) {
+				break;
+			}
+			/* we need to check the other way around only on
+			 * 2 hosts resources */
+			if (interMachine && col[j + i*howMany].measurement.timeStamp != 0) {
+				break;
+			}
+		}
+		if (j < howMany) {
+			/* we do have measurement of this host */
+			(*list)[soFar++] = i;
+		}
+	}
+
+	return (soFar);
+}
+
 static int
 AskForAllSeries(	char *resource,
 			char *opts,
 			char **hosts) {
 	int interMachine, i, j;
-	char *filter;
+	char *filter, tmp[MAX_HOST_NAME], *ptr;
 	NWSAPI_ObjectSet whereTo;
 	NWSAPI_Object obj;
 
@@ -324,7 +439,7 @@ AskForAllSeries(	char *resource,
 
 	/* let's start to allocate some memory for the filter: this
 	 * should be enough to get going. We'll realloc later on */
-	filter = MALLOC(40 + strlen(resource));
+	filter = MALLOC(40 + strlen(resource), 0);
 	if (filter == NULL) {
 		ERROR("AskForAllSeries: out of memory!!\n");
 		return 0;
@@ -349,21 +464,52 @@ AskForAllSeries(	char *resource,
 			}
 
 			/* get same memory for the filter */
-			filter = REALLOC(filter, strlen(filter) + strlen(hosts[i]) + strlen(hosts[j]) + 25);
+			filter = REALLOC(filter, strlen(filter) + strlen(hosts[i]) + strlen(hosts[j]) + 25, 0);
 			if (filter == NULL) {
 				ERROR("AskForAllSeries: out of memory\n");
 				return 0;
 			}
 			/* add the host= part */
 			strcat(filter, "(&(host=");
-			strcat(filter, hosts[i]);
-			strcat(filter, "*");
+			
+			/* thanks to our damn department, the DNS is
+			 * broken (machines reports no domain name) so we
+			 * need to add .* */
+			strcpy(tmp, hosts[i]);
+			ptr = strchr(tmp, ':');
+			if (!ptr && !strchr(tmp, '.')) {
+				/* easy: there is no port (:port) nor
+				 * domain name: we add a .* to the name */
+				strcat(tmp, ".*");
+			} else if (ptr && !strchr(tmp, '.')) {
+				/* we got a port but not a domain name:
+				 * we need to insert * in between: I'd
+				 * like to use .* but bart:8888 won't be
+				 * matched ...*/
+				memmove(ptr + 1, ptr, strlen(ptr) + 1);
+				*ptr = '*';
+			} else if (!ptr && strchr(tmp, '.')) {
+				/* we got domain but not the port: let's
+				 * add a * to match everything */
+				strcat(tmp, "*");
+			}
+			strcat(filter, tmp);
 
 			/* do we need the target= ?*/
 			if (interMachine) {
 				strcat(filter, ")(target=");
-				strcat(filter, hosts[j]);
-				strcat(filter, "*");
+				/* we need to fix the target name too! */
+				strcpy(tmp, hosts[j]);
+				ptr = strchr(tmp, ':');
+				if (!ptr && !strchr(tmp, '.')) {
+					strcat(tmp, ".*");
+				} else if (ptr && !strchr(tmp, '.')) {
+					memmove(ptr + 1, ptr, strlen(ptr) + 1);
+					*ptr = '*';
+				} else if (!ptr && strchr(tmp, '.')) {
+					strcat(tmp, "*");
+				}
+				strcat(filter, tmp);
 			}
 
 			/* close this subclause */
@@ -373,31 +519,29 @@ AskForAllSeries(	char *resource,
 	/* close the whole filter */
 	strcat(filter, ")");
 
-	/* let's query the nameserver */
-	if (!NWSAPI_GetObjects(filter, &whereTo)) {
+	/* let's query the nameserver: if the query is big the namserver
+	 * will take sometime to answer. */
+	i = GetFromNS(filter, &whereTo);
+	FREE(filter);
+
+	if (i == 0) {
 		ERROR("AskForAllSeries: failed to talk to the nameserver\n");
-		FREE(filter);
-		return 0;
 	} else {
 		/* let's add it to the todo list */
 		NWSAPI_ForEachObject(whereTo, obj) {
-			if (!NWSAPI_AddObject(&toStart, obj)) {
-				ERROR("AskForAllSeries: out of memory!\n");
-				break;
-			}
+			NWSAPI_AddObject(&toStart, obj);
 		}
 		NWSAPI_FreeObjectSet(&whereTo);
 	}
-	FREE(filter);
 
-	return 1;
+	return (i);
 }
 
 /* fills the resource, options and hosts coming from the net. Returns the
  * # of hosts, 0 on error */
 static int
 FillHosts(	Socket *sd,
-		MessageHeader header,
+		size_t dataSize,
 		char **resource,
 		char **opts,
 		char ***h) {
@@ -406,35 +550,37 @@ FillHosts(	Socket *sd,
 	int i;
 
 	/* allocate memory */
-	tmp = MALLOC(header.dataSize * sizeof(char));
+	tmp = MALLOC(dataSize * sizeof(char), 0);
 	if (tmp == NULL) {
 		return 0;
 	}
-	des.repetitions = header.dataSize;
+	des.repetitions = dataSize;
 	/* get all the data */
 	if (!RecvData(*sd, tmp, &des, 1, -1)) {
 		FREE(tmp);
 		return 0;
 	}
-	tmp[header.dataSize] = '\0';
+	tmp[dataSize - 1] = '\0';
 	ptr = tmp;
 
 	/* first is the resource name */
-	*resource = ptr;
+	*resource = strdup(ptr);
+	ptr += strlen(ptr) + 1;
 
 	/* then the options */
+	*opts = strdup(ptr);
 	ptr += strlen(ptr) + 1;
-	*opts = ptr;
-	ptr += strlen(ptr) + 1;
+
+	/* check out of memory conditions */
+	if (*resource == NULL || *opts == NULL) {
+		ABORT("FillHost: out of memory!");
+	}
 
 	/* then all the hostnames: a lot of realloc down here */
 	hosts = NULL;
-	for(i = 0; (ptr - tmp + strlen(ptr) + 1) <= header.dataSize; i++) {
+	for(i = 0; (ptr - tmp + 1) <= dataSize; i++) {
 		/* allocate space for the pointer first */
-		hosts = (char **)REALLOC(hosts, sizeof(char *)*(i + 2));
-		if (hosts == NULL) {
-			ABORT("FillHost: out of memory!");
-		}
+		hosts = (char **)REALLOC(hosts, sizeof(char *)*(i + 2), 1);
 		hosts[i] = strdup(ptr);
 		if (hosts[i] == NULL) {
 			ABORT("FillHosts: out of memory!");
@@ -442,15 +588,16 @@ FillHosts(	Socket *sd,
 		hosts[i + 1] = NULL;
 		ptr += strlen(ptr) + 1;
 	}
+	FREE(tmp);
+
 	/* check if we do indeed have hosts */
-	if (hosts == NULL) {
-		FREE(tmp);
-		return 0;
+	if (hosts != NULL) {
+		*h = hosts;
 	}
-	*h = hosts;
 
 	return i;
 }
+
 static void
 ProcessRequest(	Socket *sd,
 		MessageHeader header) {
@@ -459,6 +606,12 @@ ProcessRequest(	Socket *sd,
 	int i, j, howMany;
 	NWSAPI_ForecastCollection *col;
 	DataDescriptor howManyDes = SIMPLE_DATA(INT_TYPE, 1);
+	DataDescriptor strDes = SIMPLE_DATA(CHAR_TYPE, 0);
+	static const int sizeofForecast = sizeof(NWSAPI_ForecastCollection);
+	NWSAPI_ObjectSet objs;
+	NWSAPI_Object obj;
+
+	resource = opts = NULL;
 
 	switch(header.message) {
 	case STATE_FETCHED:
@@ -482,54 +635,109 @@ ProcessRequest(	Socket *sd,
 		break;
 
 	case GET_FORECASTS:
-		i = FillHosts(sd, header, &resource, &opts, &hosts);
+		i = FillHosts(sd, header.dataSize, &resource, &opts, &hosts);
 		if (i == 0) {
 			SendMessage(*sd, PROXY_FAILED, -1);
-			ERROR("ProcessRequest: error in reading hosts!\n");
+			ERROR("ProcessRequest: error in receiving request!\n");
 			DROP_SOCKET(sd);
+			FREE(resource);
+			FREE(opts);
 			break;
 		}
 
 		/* let's allocate room for the forecasts */
-		col = (NWSAPI_ForecastCollection *)MALLOC(sizeof(NWSAPI_ForecastCollection)*i*i);
+		col = (NWSAPI_ForecastCollection *)MALLOC(sizeofForecast*i*i, 0);
 		if (col == NULL) {
-			SendMessage(*sd, PROXY_FAILED, -1);
 			ERROR("ProcessRequest: out of memory!\n");
+			SendMessage(*sd, PROXY_FAILED, -1);
 			DROP_SOCKET(sd);
-			break;
-		}
+		} else if (!GetAllForecasts(resource, opts, hosts, i, col)) {
+			SendMessage(*sd, PROXY_FAILED, -1);
+			DROP_SOCKET(sd);
+		} else {
+			int *list, z, inter;
 
-		/* now let's get the forecats */
-		if (GetAllForecasts(resource, opts, hosts, i, col, &howMany)) {
+			inter= NWSAPI_IntermachineResource(resource);
+
+			/* let's see if we have non-known hosts, and
+			 * calculate how many forecasts we are sending */
+			howMany = CheckEmpty(inter, hosts, i, col, &list);
+			
 			/* send how many data we are shipping */
-			if (!SendMessageAndData(*sd, GOT_FORECASTS, &howMany, &howManyDes, 1, -1 )) {
+			howManyDes.repetitions = 1;
+			if (!SendMessageAndData(*sd, GOT_FORECASTS, &howMany, &howManyDes, 1, -1)) {
 				ERROR("ProcessRequest: failed to send GOT_FORECASTS\n");
 				DROP_SOCKET(sd);
-			} else for (j = 0; j < howMany; j++) {
-				/* we need to send one forecast at a time */
-				if (!SendData(*sd, &col[j], forecastCollectionDescriptor, forecastCollectionDescriptorLength, -1 )) {
-					ERROR("ProcessRequest: failed to send GOT_FORECASTS\n");
+			} else if (howMany > 0) {
+				/* we send the list only if we have
+				 * something to send */
+				howManyDes.repetitions = howMany;
+				if (!SendData(*sd, list, &howManyDes, 1, -1 )) {
+					ERROR("ProcessRequest: failed to send GOT_FORECASTS list\n");
 					DROP_SOCKET(sd);
-					break;
+				} else for (j = 0; j < howMany; j++) {
+					/* j is destination and on single
+					 * host resource cannot be > 0 */
+					if (!inter && j > 0) {
+						break;
+					}
+					for (z = 0; z < howMany; z++) {
+						int ciccio;
+
+						/* let's compute the
+						 * index we are sending */
+						if (inter && j == z) {
+							/* we don't send
+							 * the diagonal
+							 * in a 2 host
+							 * resource */
+							continue;
+						} else if (!inter) {
+							/* it's a simple
+							 * index if we
+							 * have just 1
+							 * host resource*/
+							ciccio = list[z];
+						} else {
+							ciccio = list[z] + list[j]*i;
+						}
+
+						/* we need to send one
+						 * forecast at a time */
+						if (!SendData(*sd, &col[ciccio], forecastCollectionDescriptor, forecastCollectionDescriptorLength, -1 )) {
+							ERROR("ProcessRequest: failed to send GOT_FORECASTS\n");
+							DROP_SOCKET(sd);
+							break;
+						}
+					}
+					if (z < howMany) {
+						/* got an error */
+						break;
+					}
 				}
 			}
-		} else {
-			SendMessage(*sd, PROXY_FAILED, -1);
-			DROP_SOCKET(sd);
+			FREE(list);
 		}
-		for (;i >= 0; i--) {
+
+		/* free the memory */
+		for (i--; i >= 0; i--) {
 			FREE(hosts[i]);
 		}
 		FREE(hosts);
+		FREE(resource);
+		FREE(opts);
+		FREE(col);
 		
 		break;
 
 	case START_FETCHING:
-		i = FillHosts(sd, header, &resource, &opts, &hosts);
+		i = FillHosts(sd, header.dataSize, &resource, &opts, &hosts);
 		if (i == 0) {
 			SendMessage(*sd, PROXY_FAILED, -1);
 			ERROR("ProcessRequest: error in reading hosts!\n");
 			DROP_SOCKET(sd);
+			FREE(resource);
+			FREE(opts);
 			break;
 		}
 
@@ -540,10 +748,47 @@ ProcessRequest(	Socket *sd,
 		AskForAllSeries(resource, opts, hosts);
 
 		/* done: free memory */
-		for (;i >= 0; i--) {
+		for (i-- ;i >= 0; i--) {
 			FREE(hosts[i]);
 		}
 		FREE(hosts);
+		FREE(resource);
+		FREE(opts);
+
+		break;
+
+	case START_ACTIVITY_FETCHING:
+		strDes.repetitions = header.dataSize;
+		resource = MALLOC(strDes.repetitions + 1, 1);
+		if (strDes.repetitions > 220 || !RecvData(*sd, resource, &strDes, 1, -1)) {
+			SendMessage(*sd, PROXY_FAILED, -1);
+			ERROR("ProcessRequest: error in reading clique!\n");
+			DROP_SOCKET(sd);
+			FREE(resource);
+			break;
+		}
+		resource[strDes.repetitions] = '\0';
+
+		/* let's query the nameserver for all the series relative
+		 * to this activity */
+		sprintf(name, "(activity=%s)", resource);
+		FREE(resource);
+
+		objs = NWSAPI_NewObjectSet();
+		if (GetFromNS(name, &objs)) {
+			/* let's add it to the todo list */
+			NWSAPI_ForEachObject(objs, obj) {
+				NWSAPI_AddObject(&toStart, obj);
+			}
+
+			/* no error message to the client from here */
+			SendMessage(*sd, PROXY_ACK, -1);
+		} else {
+			ERROR("ProcessRequest: failed to talk to nameserver\n");
+			SendMessage(*sd, PROXY_FAILED, -1);
+		}
+		NWSAPI_FreeObjectSet(&objs);
+
 
 		break;
 	
@@ -564,7 +809,7 @@ AddToTabList(	char **list,
 	int srcLen;
 
 	srcLen = strlen(src);
-	*list = REALLOC(*list, len + srcLen + 2);
+	*list = REALLOC(*list, len + srcLen + 2, 0);
 	if (*list == NULL) {
 		ERROR("out of memory\n");
 		return 0;
@@ -579,19 +824,11 @@ AddToTabList(	char **list,
 	return len;
 }
 
-#define SWITCHES "AN:v:p:V"
-
 void
 usage() {
-	printf("\nUsage: nws_proxy [OPTIONS]\n");
-	printf("Proxy for the Network Weather Service\n");
-	printf("\nOPTIONS can be:\n");
-	printf("\t-N nameserver       use this nameserver\n");
-	printf("\t-p port             bind to port instead of the default\n");
+	DaemonUsage("nws_proxy", "Proxy");
+	printf("\t-s                  system wide proxy (register with nameserver)\n");
 	printf("\t-A                  autofetch all the registered series\n");
-	printf("\t-v level            verbose level (up to 5)\n");
-	printf("\t-V                  print version\n");
-	printf("Report bugs to <nws@nws.cs.ucsb.edu>.\n\n");
 }
 
 
@@ -601,96 +838,35 @@ char * GenerateList(const registrations *r);
 
 int
 main (int argc, char **argv) {
-	NWSAPI_HostSpec NS;
-	int opt, verbose, len, startFetching;
-	unsigned short port;
+	int opt, startFetching, ind;
 	extern char *optarg;
 	extern int optind;
-	FILE *logFD, *errFD;
 	double now, wakeup, nextErrorCheck, nextBeatTime, nextGetOldData;
-	char *tmp, name[255], *pidFile;
+	char *tmp, name[255];
 	ObjectSet mySet;
 
 	/* default values  */
-	pidFile = NULL;
-	verbose = 2;
-	logFD = stdout;
-	errFD = stderr;
-	len = 0;
 	startFetching = 0;
-	port = NWSAPI_GetDefaultPort(NWSAPI_PROXY_HOST);
 	
-	/* get the default nameserver */
-	tmp = NWSAPI_EnvironmentValue_r("NAME_SERVER", "localhost");
-	if (tmp == NULL) {
-		ABORT("out of memory\n");
-	}
-	NS = *NWSAPI_MakeHostSpec(tmp, NWSAPI_GetDefaultPort(NWSAPI_NAME_SERVER_HOST));
-	FREE(tmp);
+	/* no error message from getopt() */
+	opterr = 0;
+
+	ind = InitializeDaemonStructure(PROXY_HOST, "sA", MyExit);
+
+	while((int)(opt = getopt(argc, argv, DaemonAllSwitches(ind))) != EOF) {
+		if (DaemonSwitch(opt, optarg, ind)) {
+			continue;
+		}
 
-	while((int)(opt = getopt(argc, argv, SWITCHES)) != EOF) {
 		switch(opt) {
-		case 'l':
-			/* open the log file */
-			logFD = fopen(optarg, "w");
-			if (logFD == NULL) {
-				printf("Couldn't open %s!\n", optarg);
-				exit(1);
-			}
-			break;
-
-		case 'e':
-			/* open the error file */
-			errFD = fopen(optarg, "w");
-			if (errFD == NULL) {
-				printf("Couldn't open %s!\n", optarg);
-				exit(1);
-			}
-			break;
-
-		case 'i':
-			/* write pid to file */
-			pidFile = strdup(optarg);
-			if (pidFile == NULL) {
-				ABORT("out of memory\n");
-			}
+		case 's':
+			local = 0;
 			break;
 
 		case 'A':
 			startFetching = 1;
 			break;
 
-		case 'p':
-			port = (unsigned short)strtol(optarg, NULL, 10);
-			break;
-
-		case 'N':
-			NS = *NWSAPI_MakeHostSpec(optarg, NWSAPI_GetDefaultPort(NWSAPI_NAME_SERVER_HOST));
-			break;
-
-		case 'V':
-			printf("nws_proxy for NWS version %s", VERSION);
-#ifdef HAVE_PTHREAD_H
-			printf(", with thread support");
-#endif
-#ifdef WITH_DEBUG
-			printf(", with debug support");
-#endif
-			printf("\n\n");
-			exit(0);
-			break;
-
-		case 'v':
-			verbose = (unsigned short)atol(optarg);
-			break;
-
-		case '?':
-			if (optopt == 'v') {
-				/* using the first level */
-				verbose = 1;
-				break;
-			}
-
 		default:
 			usage();
 			exit(1);
@@ -698,28 +874,14 @@ main (int argc, char **argv) {
 		}
 	}
 
-	/* let's set the verbose evel */
-	SetDiagnosticLevel(verbose, errFD, logFD);
-
-	/* proud to serve */
-	fclose(stdin);
-
-	if (!EstablishAnEar(port, port, NULL, NULL)) {
-		ABORT("failed to establish an ear!\n");
+	/* if we are local we only open a listening port, otherwise we do
+	 * the full registration with our nameserver */
+	if (!InitializeDaemon(local, 1, 0, ind)) {
+		exit(-1);
 	}
 
-	/* now that we've got the port, we can print the pid into the *
-	 * pid file. We thus avoid to override pid files of running *
-	 * nameservers */
-	if (pidFile != NULL) {
-		FILE *pidfile = fopen(pidFile, "w");
-		if (!pidfile) {
-			ABORT1("Can't write pidfile %s\n", pidFile);
-		}
-		fprintf(pidfile, "%d", (int)getpid());
-		fclose(pidfile);
-		free(pidFile);
-	}
+	/* we want to do relax matching */
+	NWSAPI_RelaxHostNameCheck(1);
 
 	/* initialize the series structure */
 	GetNWSLock(&lock);
@@ -730,21 +892,12 @@ main (int argc, char **argv) {
 	upTo = NULL;
 	lost = getOld = NULL;
 	lostLen = getOldLen = 0;
-
-	/* let's set the nameServer */
-	if (!NWSAPI_UseNameServer(&NS)) {
-		WARN2("%s:%d is not a valid nameserver!\n", NS.machineName, NS.machinePort);
-	} else if (startFetching) {
-		/* let's get all the series names */
-		if (!NWSAPI_GetObjectsTimeout("(objectclass=nwsSeries)", &toStart, 0)) {
-			WARN("main: cannot get series!\n");
-		}
-	}
 	ReleaseNWSLock(&lock);
 
 	RegisterListener(STATE_FETCHED, "STATE_FETCHED,", &ProcessRequest);
 	RegisterListener(GET_FORECASTS, "GET_FORECASTS,", &ProcessRequest);
 	RegisterListener(START_FETCHING, "START_FETCHING,", &ProcessRequest);
+	RegisterListener(START_ACTIVITY_FETCHING, "START_ACTIVITY_FETCHING,", &ProcessRequest);
 
 	/* main service loop */
 	wakeup = nextErrorCheck = nextBeatTime = nextGetOldData = 0;
@@ -753,6 +906,9 @@ main (int argc, char **argv) {
 
 		/* we might want to register with the nameserver here */
 		if (nextBeatTime < now) {
+			if (!local) {
+				RegisterHost(DEFAULT_HOST_BEAT * 2);
+			}
 			LOG1("main: we have %d series\n", series->howMany);
 
 			/* some feedback */
@@ -775,6 +931,19 @@ main (int argc, char **argv) {
 			nextErrorCheck = now + 20;
 		}
 
+		/* now, if we've been asked to fetch all the series from
+		 * a nameserver (-A) we do so periodically to be sure we
+		 * keep getting new series */
+		if (startFetching > 0 && startFetching < now) {
+			startFetching = now + 60*60;
+
+			/* let's get all the series names */
+			if (!GetFromNS("(objectclass=nwsSeries)", &toStart)) {
+				WARN("main: cannot get series!\n");
+			}
+		}
+
+
 		if (nextBeatTime > nextErrorCheck) {
 			wakeup = nextErrorCheck;
 		} else {
@@ -793,15 +962,10 @@ main (int argc, char **argv) {
 			if (!AddSingleForecast(upTo)) {
 				WARN("failed to add forecaster\n");
 			} else {
-				if (!AddObject(&mySet, upTo)) {
-					ABORT("out of memory!\n");
-				}
+				NWSAPI_AddObject(&mySet, upTo);
 
 				/* add the series to the restart list */
 				tmp = NwsAttributeValue_r(FindNwsAttribute(upTo, "name"));
-				if (tmp == NULL) {
-					ABORT("out of memory\n");
-				}
 				getOldLen = AddToTabList(&getOld, getOldLen, tmp);
 				if (getOldLen == 0) {
 					ABORT("out of memory!\n");
@@ -825,14 +989,9 @@ main (int argc, char **argv) {
 		 * are using the same now, hence we do either the new
 		 * series or we rescue the lost one. */
 		while (lostLen > 0 && (CurrentTime() - now) < 1) {
+			/* get the next series and remove it from the
+			 * todo list */
 			GETWORD(name, lost, (const char **)&tmp);
-			if (!SearchForName(series, name, 1, &opt)) {
-				WARN1("non existent series (%s)??\n", name);
-				continue;
-			}
-			NWSAPI_AddObject(&mySet, series->vals[opt]);
-
-			/* remove the series we worked on */
 			memmove(lost, tmp, lostLen - (tmp - lost) + 1);
 			lostLen -= (tmp - lost);
 			if (lostLen == 0) {
@@ -842,6 +1001,14 @@ main (int argc, char **argv) {
 				/* let's come back quickly */
 				wakeup = -1;
 			}
+
+			/* now let's add it to the working list */
+			if (!SearchForName(series, name, 1, &opt)) {
+				WARN1("non existent series (%s)??\n", name);
+			} else {
+				NWSAPI_AddObject(&mySet, series->vals[opt]);
+			}
+
 		}
 		/* let's start autofetching */
 		if (strlen(mySet) > 5) {
