@@ -1,4 +1,4 @@
-/* $Id: nws_state.c,v 1.55 2004/09/28 18:40:24 graziano Exp $ */
+/* $Id: nws_state.c,v 1.58 2005/05/24 19:38:25 graziano Exp $ */
 
 #include "config_nws.h"
 
@@ -103,14 +103,11 @@ InitStateModule(	int cacheNum,
 	}
 	if (dir == NULL || strlen(dir) == 0) {
 		WARN("InitStateModule: empty direcotry: using current\n");
-		memDir = (char *)MALLOC(3);
+		memDir = (char *)MALLOC(3, 1);
 		memDir[0] = '.';
 		memDir[1] = '\0';
 	} else {
-		memDir = (char *)MALLOC(strlen(dir) + 2);
-		if (memDir == NULL) {
-			ABORT("InitStateModule: out of memory\n");
-		}
+		memDir = (char *)MALLOC(strlen(dir) + 2, 1);
 		memcpy(memDir, dir, strlen(dir));
 		memDir[strlen(dir)] = '\0';
 	}
@@ -264,8 +261,7 @@ WriteStateRec(	FILE *fd,
 		double seqNo,
 		double timeOut,
 		const char *data) {
-	size_t clientSize;
-	int i;
+	int clientSize;
 	char padded[MAX_RECORD_SIZE + 1];
 
 	if (fseek(fd, HEADER_LEN + recSize * writeIndex, SEEK_SET) < 0) {
@@ -273,10 +269,12 @@ WriteStateRec(	FILE *fd,
 	}
 
 	clientSize = recSize - OVERHEAD_LEN - 1;
-	i = strlen(data);
-	memcpy(padded, data, i);
-	if (i < clientSize) {
-		memset(padded, ' ', clientSize - i);
+	if (clientSize <= 0) {
+		FAIL("WriteStateRec: too short record!\n");
+	}
+	memcpy(padded, data, clientSize);
+	if (recSize < clientSize) {
+		memset(padded, ' ', clientSize - recSize);
 	}
 	padded[clientSize] = '\0';
 	fprintf(fd, STATE_FORMAT, timeStamp, seqNo, timeOut, padded);
@@ -621,7 +619,7 @@ ReadOldStates() {
 	}
 
 	len = 0;
-	tmp = MALLOC(1);
+	tmp = MALLOC(1, 0);
 	if (tmp == NULL) {
 		ERROR("ReadOldStates: out of memory\n");
 		closedir(dir);
@@ -642,7 +640,7 @@ ReadOldStates() {
 			tmp[len++] = '\n';	/* add a marker for new entry */
 			tmp[len] = '\0';
 		}
-		tmp = (char*)REALLOC(tmp, len + strlen(entry->d_name) + 2);
+		tmp = (char*)REALLOC(tmp, len + strlen(entry->d_name) + 2, 1);
 		memcpy(tmp + len, entry->d_name, strlen(entry->d_name) + 1);
 		len += strlen(entry->d_name);
 		howMany++;
@@ -684,7 +682,7 @@ CheckFileName(	const char *name) {
 	}
 	j = strlen(name);
 
-	tmp = MALLOC(i + j + 1);
+	tmp = MALLOC(i + j + 1, 0);
 	if (tmp == NULL) {
 		ERROR("CheckFileName: out of memory\n");
 		return 0;
@@ -712,39 +710,32 @@ CheckFileName(	const char *name) {
  * #idle# seconds. Returns 1 if succesful.
  */
 int
-CleanLocalStates(unsigned long idle) {
-#ifdef HAVE_DIRENT_H
-	struct dirent *entry;
-	DIR *directory;
+CleanLocalStates(	const char *series,
+			unsigned long idle) {
 	time_t expiration;
-	char filePath[255 + 1];
+	char *filePath;
 	struct stat fileStat;
-	char *namePlace;
-							                                                                                
-	directory = opendir(memDir);
-	if(directory == NULL) {
-		FAIL1("CleanLocalStates: unable to open %s\n", memDir);
+	int ret;
+
+	if (series == NULL || strlen(series) <= 1) {
+		ERROR("CleanLocalStates: NULL parameter!\n");
+		return 0;
 	}
-	
+
+	ret = 0;
 	expiration = (time_t)CurrentTime() - (time_t)idle;
-	SAFESTRCPY(filePath, memDir);
-	namePlace = filePath + strlen(filePath);
-	
-	while((entry = readdir(directory)) != NULL) {
-		strcpy(namePlace, entry->d_name);
-		if(stat(filePath, &fileStat) != 0) {
-			WARN1("CleanLocalStates: unable to state file %s\n", filePath);
-		} else if(fileStat.st_mtime < expiration) {
-			LOG2("CleanLocalStates: deleting %s, last modified %d\n",
-			filePath, fileStat.st_mtime);
-			(void)unlink(filePath);
-		}
+	filePath = MALLOC(strlen(memDir) + strlen(series) + 1, 1);
+	sprintf(filePath, "%s%s", memDir, series);
+	if(stat(filePath, &fileStat) != 0) {
+		WARN1("CleanLocalStates: unable to state file %s\n", filePath);
+	} else if(fileStat.st_mtime < expiration) {
+		LOG2("CleanLocalStates: deleting %s, last modified %d\n", filePath, fileStat.st_mtime);
+		(void)unlink(filePath);
+		ret = 1;
 	}
-	(void)closedir(directory);
-#else
-	ERROR("CleanLocalStates: NO dirent.h, can't read directory!\n");
-#endif
-	return(1);
+	free(filePath);
+
+	return (ret);
 }
 
 
